@@ -35,18 +35,25 @@
 #include <gio/gio.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
+#ifdef HAVE_VTE
 #include <vte/vte.h>
+#endif
 #include <canberra-gtk.h>
 
 #include "gcm-calibrate-argyll.h"
-#include "gcm-colorimeter.h"
+#include "gcm-sensor-client.h"
 #include "gcm-utils.h"
-#include "gcm-screen.h"
+#include "gcm-x11-screen.h"
 #include "gcm-print.h"
+#include "gcm-xyz.h"
 #include "gcm-calibrate-dialog.h"
 
 #include "egg-debug.h"
 
+#define FIXED_ARGYLL
+
+//#define USE_DOUBLE_DENSITY
+
 static void     gcm_calibrate_argyll_finalize	(GObject     *object);
 
 #define GCM_CALIBRATE_ARGYLL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_CALIBRATE_ARGYLL, GcmCalibrateArgyllPrivate))
@@ -72,7 +79,7 @@ struct _GcmCalibrateArgyllPrivate
 	GcmCalibrateDialog		*calibrate_dialog;
 	pid_t				 child_pid;
 	GtkResponseType			 response;
-	GcmScreen			*screen;
+	GcmX11Screen			*screen;
 	glong				 vte_previous_row;
 	glong				 vte_previous_col;
 	gboolean			 already_on_window;
@@ -80,13 +87,8 @@ struct _GcmCalibrateArgyllPrivate
 	GcmCalibrateArgyllState		 state;
 	GcmPrint			*print;
 	const gchar			*argyllcms_ok;
-	guint				 terminal_child_exited_id;
-	guint				 terminal_cursor_moved_id;
-};
-
-enum {
-	PROP_0,
-	PROP_LAST
+	gboolean			 done_spot_read;
+	guint				 keypress_id;
 };
 
 G_DEFINE_TYPE (GcmCalibrateArgyll, gcm_calibrate_argyll, GCM_TYPE_CALIBRATE)
@@ -151,12 +153,12 @@ static guint
 gcm_calibrate_argyll_printer_get_patches (GcmCalibrateArgyll *calibrate_argyll)
 {
 	guint patches = 180;
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 	GcmCalibratePrecision precision;
 
 	/* we care about the kind */
 	g_object_get (calibrate_argyll,
-		      "colorimeter-kind", &colorimeter_kind,
+		      "sensor-kind", &sensor_kind,
 		      "precision", &precision,
 		      NULL);
 
@@ -167,64 +169,68 @@ gcm_calibrate_argyll_printer_get_patches (GcmCalibrateArgyll *calibrate_argyll)
 	else if (precision == GCM_CALIBRATE_PRECISION_LONG)
 		patches = 360;
 
+#ifdef USE_DOUBLE_DENSITY
 	/* using double density, so we can double the patch count */
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI ||
-	    colorimeter_kind == GCM_COLORIMETER_KIND_SPECTRO_SCAN) {
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI ||
+	    sensor_kind == GCM_SENSOR_KIND_SPECTRO_SCAN) {
 		patches *= 2;
 	}
+#endif
 
 	return patches;
 }
 
+#ifdef HAVE_VTE
 /**
- * gcm_calibrate_argyll_get_colorimeter_image_attach:
+ * gcm_calibrate_argyll_get_sensor_image_attach:
  **/
 static const gchar *
-gcm_calibrate_argyll_get_colorimeter_image_attach (GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_get_sensor_image_attach (GcmCalibrateArgyll *calibrate_argyll)
 {
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 
-	g_object_get (calibrate_argyll, "colorimeter-kind", &colorimeter_kind, NULL);
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_HUEY)
+	g_object_get (calibrate_argyll, "sensor-kind", &sensor_kind, NULL);
+	if (sensor_kind == GCM_SENSOR_KIND_HUEY)
 		return "huey-attach.svg";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI)
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI)
 		return "munki-attach.svg";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_SPYDER)
+	if (sensor_kind == GCM_SENSOR_KIND_SPYDER)
 		return "spyder-attach.svg";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLORIMTRE_HCFR)
+	if (sensor_kind == GCM_SENSOR_KIND_COLORIMTRE_HCFR)
 		return "hcfr-attach.svg";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_I1_PRO)
+	if (sensor_kind == GCM_SENSOR_KIND_I1_PRO)
 		return "i1-attach.svg";
 	return NULL;
 }
 
 /**
- * gcm_calibrate_argyll_get_colorimeter_image_calibrate:
+ * gcm_calibrate_argyll_get_sensor_image_calibrate:
  **/
 static const gchar *
-gcm_calibrate_argyll_get_colorimeter_image_calibrate (GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_get_sensor_image_calibrate (GcmCalibrateArgyll *calibrate_argyll)
 {
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 
-	g_object_get (calibrate_argyll, "colorimeter-kind", &colorimeter_kind, NULL);
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI)
+	g_object_get (calibrate_argyll, "sensor-kind", &sensor_kind, NULL);
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI)
 		return "munki-calibrate.svg";
 	return NULL;
 }
 
 /**
- * gcm_calibrate_argyll_get_colorimeter_image_screen:
+ * gcm_calibrate_argyll_get_sensor_image_screen:
  **/
 static const gchar *
-gcm_calibrate_argyll_get_colorimeter_image_screen (GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_get_sensor_image_screen (GcmCalibrateArgyll *calibrate_argyll)
 {
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 
-	g_object_get (calibrate_argyll, "colorimeter-kind", &colorimeter_kind, NULL);
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI)
+	g_object_get (calibrate_argyll, "sensor-kind", &sensor_kind, NULL);
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI)
 		return "munki-screen.svg";
 	return NULL;
 }
+#endif
 
 /**
  * gcm_calibrate_argyll_get_display:
@@ -358,9 +364,8 @@ static gboolean
 gcm_calibrate_argyll_fork_command (GcmCalibrateArgyll *calibrate_argyll, gchar **argv, GError **error)
 {
 	gboolean ret = FALSE;
-#if VTE_CHECK_VERSION(0,25,0)
+#ifdef HAVE_VTE
 	const gchar *envp[] = { "ARGYLL_NOT_INTERACTIVE", NULL };
-#endif
 	const gchar *working_directory;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 
@@ -370,34 +375,29 @@ gcm_calibrate_argyll_fork_command (GcmCalibrateArgyll *calibrate_argyll, gchar *
 
 	/* try to run */
 	working_directory = gcm_calibrate_get_working_path (GCM_CALIBRATE (calibrate_argyll));
-
-#if VTE_CHECK_VERSION(0,25,0)
 	ret = vte_terminal_fork_command_full (VTE_TERMINAL(priv->terminal),
 					      VTE_PTY_DEFAULT,
 					      working_directory,
 					      argv, (gchar**)envp,
+#ifdef FIXED_ARGYLL
+					      0,
+#else
 					      G_SPAWN_FILE_AND_ARGV_ZERO,
+#endif
 					      NULL, NULL,
 					      &priv->child_pid, error);
-#else
-	priv->child_pid = vte_terminal_fork_command (VTE_TERMINAL(priv->terminal),
-						     argv[0], &argv[1], NULL,
-						     working_directory,
-						     FALSE, FALSE, FALSE);
-	ret = (priv->child_pid > 0);
-	if (!ret) {
-		g_set_error (error,
-			     GCM_CALIBRATE_ERROR,
-			     GCM_CALIBRATE_ERROR_USER_ABORT,
-			     "failed to spawn %s", argv[0]);
-	}
-#endif
 	if (!ret)
 		goto out;
 
 	/* we're running */
 	priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 out:
+#else
+	g_set_error_literal (error,
+			     GCM_CALIBRATE_ERROR,
+			     GCM_CALIBRATE_ERROR_INTERNAL,
+			     "no VTE support");
+#endif
 	return ret;
 }
 
@@ -412,7 +412,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 	gchar kind;
 	gchar *command = NULL;
 	gchar **argv = NULL;
-	GnomeRROutput *output;
+	GcmX11Output *output;
 	GPtrArray *array = NULL;
 	gchar *basename = NULL;
 	gchar *output_name = NULL;
@@ -440,7 +440,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 	}
 
 	/* get the device */
-	output = gcm_screen_get_output_by_name (priv->screen, output_name, error);
+	output = gcm_x11_screen_get_output_by_name (priv->screen, output_name, error);
 	if (output == NULL) {
 		ret = FALSE;
 		goto out;
@@ -464,7 +464,9 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup ("-ql"));
 	g_ptr_array_add (array, g_strdup ("-m"));
@@ -492,6 +494,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -503,6 +506,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -555,7 +559,9 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	if (priv->done_calibrate)
 		g_ptr_array_add (array, g_strdup ("-N"));
@@ -580,6 +586,7 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -591,6 +598,7 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -643,7 +651,9 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	if (device_kind == GCM_DEVICE_KIND_PRINTER) {
 		/* print RGB */
@@ -683,6 +693,7 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -694,6 +705,7 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -748,7 +760,9 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-d%i", priv->display));
 	g_ptr_array_add (array, g_strdup_printf ("-y%c", kind));
@@ -775,6 +789,7 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -786,6 +801,7 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -859,7 +875,9 @@ gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-A%s", manufacturer));
 	g_ptr_array_add (array, g_strdup_printf ("-M%s", model));
@@ -888,6 +906,7 @@ gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -899,7 +918,7 @@ gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
-
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -1080,7 +1099,9 @@ gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GErro
 	filename = g_strdup_printf ("%s.tif", basename);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup ("-p"));
 	g_ptr_array_add (array, g_strdup ("-a"));
@@ -1107,6 +1128,7 @@ gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GErro
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -1118,6 +1140,7 @@ gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GErro
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	g_free (filename);
 	g_free (command);
@@ -1194,7 +1217,9 @@ gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argy
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-A%s", manufacturer));
 	g_ptr_array_add (array, g_strdup_printf ("-M%s", model));
@@ -1228,6 +1253,7 @@ gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argy
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -1239,6 +1265,7 @@ gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argy
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -1414,30 +1441,142 @@ out:
 }
 
 /**
- * gcm_calibrate_argyll_get_colorimeter_target:
+ * gcm_calibrate_argyll_spotread_read_chart:
+ **/
+static gboolean
+gcm_calibrate_argyll_spotread_read_chart (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+{
+	gboolean ret = TRUE;
+	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	gchar *command = NULL;
+	gchar **argv = NULL;
+	GPtrArray *array = NULL;
+	const gchar *title;
+	const gchar *message;
+
+	/* get correct name of the command */
+	command = gcm_calibrate_argyll_get_tool_filename ("spotread", error);
+	if (command == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+
+	/* TRANSLATORS: title, setting up the photospectromiter */
+	title = _("Setting up device");
+	/* TRANSLATORS: dialog message */
+	message = _("Setting up the device to read a spot color…");
+
+	/* push new messages into the UI */
+	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
+	gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, FALSE);
+	gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, TRUE);
+
+	/* reset flag so we exit after the single spotread */
+	priv->done_spot_read = FALSE;
+
+	/* argument array */
+	array = g_ptr_array_new_with_free_func (g_free);
+
+	/* setup the command */
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
+	if (priv->done_calibrate)
+		g_ptr_array_add (array, g_strdup ("-N"));
+	argv = gcm_utils_ptr_array_to_strv (array);
+	gcm_calibrate_argyll_debug_argv (command, argv);
+
+	/* start up the command */
+	ret = gcm_calibrate_argyll_fork_command (calibrate_argyll, argv, error);
+	if (!ret)
+		goto out;
+
+	/* wait until finished */
+	g_main_loop_run (priv->loop);
+
+	/* get result */
+	if (priv->response == GTK_RESPONSE_CANCEL) {
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_USER_ABORT,
+				     "calibration was cancelled");
+		ret = FALSE;
+		goto out;
+	}
+#ifdef HAVE_VTE
+	if (priv->response == GTK_RESPONSE_REJECT) {
+		gchar *vte_text;
+		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
+		g_set_error (error,
+			     GCM_CALIBRATE_ERROR,
+			     GCM_CALIBRATE_ERROR_INTERNAL,
+			     "command failed to run successfully: %s", vte_text);
+		g_free (vte_text);
+		ret = FALSE;
+		goto out;
+	}
+#endif
+out:
+	if (array != NULL)
+		g_ptr_array_unref (array);
+	g_free (command);
+	g_strfreev (argv);
+	return ret;
+}
+
+/**
+ * gcm_calibrate_argyll_spotread:
+ **/
+static gboolean
+gcm_calibrate_argyll_spotread (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
+{
+	GcmCalibrateArgyll *calibrate_argyll = GCM_CALIBRATE_ARGYLL(calibrate);
+	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	gboolean ret;
+
+	/* set modal windows up correctly */
+	gcm_calibrate_dialog_set_move_window (priv->calibrate_dialog, FALSE);
+	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
+
+	/* step 3 */
+	ret = gcm_calibrate_argyll_spotread_read_chart (calibrate_argyll, error);
+	if (!ret)
+		goto out;
+
+	/* step 5 */
+	ret = gcm_calibrate_argyll_remove_temp_files (calibrate_argyll, error);
+	if (!ret)
+		goto out;
+out:
+	return ret;
+}
+
+/**
+ * gcm_calibrate_argyll_get_sensor_target:
  **/
 static const gchar *
-gcm_calibrate_argyll_get_colorimeter_target (GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_get_sensor_target (GcmCalibrateArgyll *calibrate_argyll)
 {
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 
 	g_object_get (calibrate_argyll,
-		      "colorimeter-kind", &colorimeter_kind,
+		      "sensor-kind", &sensor_kind,
 		      NULL);
 
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_DTP20)
+	if (sensor_kind == GCM_SENSOR_KIND_DTP20)
 		return "20";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_DTP22)
+	if (sensor_kind == GCM_SENSOR_KIND_DTP22)
 		return "22";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_DTP41)
+	if (sensor_kind == GCM_SENSOR_KIND_DTP41)
 		return "41";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_DTP51)
+	if (sensor_kind == GCM_SENSOR_KIND_DTP51)
 		return "51";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_SPECTRO_SCAN)
+	if (sensor_kind == GCM_SENSOR_KIND_SPECTRO_SCAN)
 		return "SS";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_I1_PRO)
+	if (sensor_kind == GCM_SENSOR_KIND_I1_PRO)
 		return "i1";
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI)
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI)
 		return "CM";
 	return NULL;
 }
@@ -1456,12 +1595,12 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 	gchar *basename = NULL;
 	const gchar *title;
 	const gchar *message;
-	GcmColorimeterKind colorimeter_kind;
+	GcmSensorKind sensor_kind;
 
 	/* get shared data */
 	g_object_get (calibrate_argyll,
 		      "basename", &basename,
-		      "colorimeter-kind", &colorimeter_kind,
+		      "sensor-kind", &sensor_kind,
 		      NULL);
 
 	/* get correct name of the command */
@@ -1486,17 +1625,21 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
 	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 
 	/* target instrument */
-	g_ptr_array_add (array, g_strdup_printf ("-i%s", gcm_calibrate_argyll_get_colorimeter_target (calibrate_argyll)));
+	g_ptr_array_add (array, g_strdup_printf ("-i%s", gcm_calibrate_argyll_get_sensor_target (calibrate_argyll)));
 
+#ifdef USE_DOUBLE_DENSITY
 	/* use double density */
-	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLOR_MUNKI ||
-	    colorimeter_kind == GCM_COLORIMETER_KIND_SPECTRO_SCAN) {
+	if (sensor_kind == GCM_SENSOR_KIND_COLOR_MUNKI ||
+	    sensor_kind == GCM_SENSOR_KIND_SPECTRO_SCAN) {
 		g_ptr_array_add (array, g_strdup ("-h"));
 	}
+#endif
 
 	/* 8 bit TIFF 300 dpi */
 	g_ptr_array_add (array, g_strdup ("-t300"));
@@ -1525,6 +1668,7 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#ifdef HAVE_VTE
 	if (priv->response == GTK_RESPONSE_REJECT) {
 		gchar *vte_text;
 		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
@@ -1536,6 +1680,7 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 		ret = FALSE;
 		goto out;
 	}
+#endif
 out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
@@ -2071,6 +2216,7 @@ out:
 	return ret;
 }
 
+#ifdef HAVE_VTE
 /**
  * gcm_calibrate_argyll_exit_cb:
  **/
@@ -2119,7 +2265,12 @@ gcm_calibrate_argyll_interaction_attach (GcmCalibrateArgyll *calibrate_argyll)
 	/* different tools assume the device is not on the screen */
 	if (priv->already_on_window) {
 		egg_debug ("VTE: already on screen so faking keypress");
-		g_timeout_add_seconds (1, (GSourceFunc) gcm_calibrate_argyll_timeout_cb, calibrate_argyll);
+		priv->keypress_id = g_timeout_add_seconds (1,
+							   (GSourceFunc) gcm_calibrate_argyll_timeout_cb,
+							   calibrate_argyll);
+#if GLIB_CHECK_VERSION(2,25,8)
+		g_source_set_name_by_id (priv->keypress_id, "[GcmCalibrateArgyll] keypress faker");
+#endif
 		goto out;
 	}
 
@@ -2127,7 +2278,7 @@ gcm_calibrate_argyll_interaction_attach (GcmCalibrateArgyll *calibrate_argyll)
 	title = _("Please attach instrument");
 
 	/* get the image, if we have one */
-	filename = gcm_calibrate_argyll_get_colorimeter_image_attach (calibrate_argyll);
+	filename = gcm_calibrate_argyll_get_sensor_image_attach (calibrate_argyll);
 
 	/* different messages with or without image */
 	if (filename != NULL) {
@@ -2185,7 +2336,7 @@ gcm_calibrate_argyll_interaction_calibrate (GcmCalibrateArgyll *calibrate_argyll
 	egg_debug ("blocking waiting for user input: %s", title);
 
 	/* get the image, if we have one */
-	filename = gcm_calibrate_argyll_get_colorimeter_image_calibrate (calibrate_argyll);
+	filename = gcm_calibrate_argyll_get_sensor_image_calibrate (calibrate_argyll);
 
 	if (filename != NULL) {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor, and we're showing a picture */
@@ -2214,6 +2365,9 @@ gcm_calibrate_argyll_interaction_calibrate (GcmCalibrateArgyll *calibrate_argyll
 	/* assume it's no longer on the window */
 	priv->already_on_window = FALSE;
 
+	/* assume it was done correctly */
+	priv->done_calibrate = TRUE;
+
 	/* set state */
 	priv->argyllcms_ok = " ";
 	priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
@@ -2237,14 +2391,14 @@ gcm_calibrate_argyll_interaction_surface (GcmCalibrateArgyll *calibrate_argyll)
 	egg_debug ("blocking waiting for user input: %s", title);
 
 	/* get the image, if we have one */
-	filename = gcm_calibrate_argyll_get_colorimeter_image_screen (calibrate_argyll);
+	filename = gcm_calibrate_argyll_get_sensor_image_screen (calibrate_argyll);
 
 	if (filename != NULL) {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor, and we're showing a picture */
-		message = _("Please set the measuring instrument to screen mode like the image below, and ensure it is attached to the screen.");
+		message = _("Please set the measuring instrument to screen mode like the image below.");
 	} else {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor */
-		message = _("Please set the measuring instrument to screen mode, and ensure it is attached to the screen.");
+		message = _("Please set the measuring instrument to screen mode.");
 	}
 
 	/* push new messages into the UI */
@@ -2266,9 +2420,6 @@ gcm_calibrate_argyll_interaction_surface (GcmCalibrateArgyll *calibrate_argyll)
 	/* assume it's no longer on the window */
 	priv->already_on_window = FALSE;
 
-	/* assume it was done correctly */
-	priv->done_calibrate = TRUE;
-
 	/* set state */
 	priv->argyllcms_ok = " ";
 	priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
@@ -2361,6 +2512,27 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		goto out;
 	}
 
+	/* spot read result */
+	found = g_strstr_len (line, -1, "Result is XYZ");
+	if (found != NULL) {
+		GcmXyz *xyz;
+		egg_warning ("line=%s", line);
+		split = g_strsplit (line, " ", -1);
+		xyz = gcm_xyz_new ();
+		g_object_set (xyz,
+			      "cie-x", g_ascii_strtod (split[4], NULL),
+			      "cie-y", g_ascii_strtod (split[5], NULL),
+			      "cie-z", g_ascii_strtod (split[6], NULL),
+			      NULL);
+		g_object_set (calibrate_argyll,
+			      "xyz", xyz,
+			      NULL);
+		priv->done_spot_read = TRUE;
+		gcm_calibrate_dialog_response (priv->calibrate_dialog, GTK_RESPONSE_CANCEL);
+		g_object_unref (xyz);
+		goto out;
+	}
+
 	/* error */
 	found = g_strstr_len (line, -1, "Error - ");
 	if (found != NULL) {
@@ -2480,6 +2652,38 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		goto out;
 	}
 
+	/* reading spot */
+	if (g_str_has_prefix (line, "Place instrument on spot to be measured")) {
+		if (!priv->done_spot_read)
+			vte_terminal_feed_child (VTE_TERMINAL(priv->terminal), " ", 1);
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+		goto out;
+	}
+
+
+	/* reading strip */
+	if (g_str_has_prefix (line, "Spot read failed due to misread")) {
+
+		/* TRANSLATORS: title, the calibration failed */
+		title = _("Device Error");
+
+		/* TRANSLATORS: message, the sample was not read correctly */
+		message = _("The device could not measure the color spot correctly.");
+
+		/* push new messages into the UI */
+		gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
+		gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, TRUE);
+		gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, TRUE);
+
+		/* TRANSLATORS: button */
+		gcm_calibrate_dialog_set_button_ok_id (priv->calibrate_dialog, _("Retry"));
+
+		/* set state */
+		priv->argyllcms_ok = " ";
+		priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
+		goto out;
+	}
+
 	/* reading strip */
 	if (g_str_has_prefix (line, "Ready to read strip pass ")) {
 
@@ -2520,7 +2724,9 @@ out:
 		g_string_free (string, TRUE);
 	return ret;
 }
+#endif
 
+#ifdef HAVE_VTE
 /**
  * gcm_calibrate_argyll_selection_func_cb:
  **/
@@ -2571,6 +2777,7 @@ gcm_calibrate_argyll_cursor_moved_cb (VteTerminal *terminal, GcmCalibrateArgyll
 	g_free (output);
 	g_strfreev (split);
 }
+#endif
 
 /**
  * gcm_calibrate_argyll_response_cb:
@@ -2589,7 +2796,9 @@ gcm_calibrate_argyll_response_cb (GtkWidget *widget, GtkResponseType response, G
 		/* send input if waiting */
 		if (priv->state == GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN) {
 			egg_debug ("sending '%s' to argyll", priv->argyllcms_ok);
+#ifdef HAVE_VTE
 			vte_terminal_feed_child (VTE_TERMINAL(priv->terminal), priv->argyllcms_ok, 1);
+#endif
 			gcm_calibrate_dialog_pop (priv->calibrate_dialog);
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
@@ -2609,7 +2818,9 @@ gcm_calibrate_argyll_response_cb (GtkWidget *widget, GtkResponseType response, G
 		/* send input if waiting */
 		if (priv->state == GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN) {
 			egg_debug ("sending 'Q' to argyll");
+#ifdef HAVE_VTE
 			vte_terminal_feed_child (VTE_TERMINAL(priv->terminal), "Q", 1);
+#endif
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
 
@@ -2619,6 +2830,9 @@ gcm_calibrate_argyll_response_cb (GtkWidget *widget, GtkResponseType response, G
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
 
+		/* hide the window */
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+
 		/* stop loop */
 		if (g_main_loop_is_running (priv->loop))
 			g_main_loop_quit (priv->loop);
@@ -2694,6 +2908,7 @@ gcm_calibrate_argyll_class_init (GcmCalibrateArgyllClass *klass)
 	parent_class->calibrate_display = gcm_calibrate_argyll_display;
 	parent_class->calibrate_device = gcm_calibrate_argyll_device;
 	parent_class->calibrate_printer = gcm_calibrate_argyll_printer;
+	parent_class->calibrate_spotread = gcm_calibrate_argyll_spotread;
 
 	g_type_class_add_private (klass, sizeof (GcmCalibrateArgyllPrivate));
 }
@@ -2720,19 +2935,19 @@ gcm_calibrate_argyll_init (GcmCalibrateArgyll *calibrate_argyll)
 			  G_CALLBACK (gcm_calibrate_argyll_response_cb), calibrate_argyll);
 
 	/* get screen */
-	calibrate_argyll->priv->screen = gcm_screen_new ();
+	calibrate_argyll->priv->screen = gcm_x11_screen_new ();
 
 	/* add vte widget */
+#ifdef HAVE_VTE
 	calibrate_argyll->priv->terminal = vte_terminal_new ();
 	vte_terminal_set_size (VTE_TERMINAL(calibrate_argyll->priv->terminal), 80, 10);
-	calibrate_argyll->priv->terminal_child_exited_id =
-		g_signal_connect (calibrate_argyll->priv->terminal, "child-exited",
-				G_CALLBACK (gcm_calibrate_argyll_exit_cb), calibrate_argyll);
-	calibrate_argyll->priv->terminal_cursor_moved_id =
-		g_signal_connect (calibrate_argyll->priv->terminal, "cursor-moved",
-				G_CALLBACK (gcm_calibrate_argyll_cursor_moved_cb), calibrate_argyll);
+	g_signal_connect (calibrate_argyll->priv->terminal, "child-exited",
+			  G_CALLBACK (gcm_calibrate_argyll_exit_cb), calibrate_argyll);
+	g_signal_connect (calibrate_argyll->priv->terminal, "cursor-moved",
+			  G_CALLBACK (gcm_calibrate_argyll_cursor_moved_cb), calibrate_argyll);
 	gcm_calibrate_dialog_pack_details (calibrate_argyll->priv->calibrate_dialog,
 					   calibrate_argyll->priv->terminal);
+#endif
 }
 
 /**
@@ -2753,13 +2968,22 @@ gcm_calibrate_argyll_finalize (GObject *object)
 		g_main_loop_run (priv->loop);
 	}
 
-	/* disconnect */
-	g_signal_handler_disconnect (priv->terminal, priv->terminal_child_exited_id);
-	g_signal_handler_disconnect (priv->terminal, priv->terminal_cursor_moved_id);
+#ifdef HAVE_VTE
+	/* we don't care if the VTE widget redraws now */
+	g_signal_handlers_disconnect_by_func (calibrate_argyll->priv->terminal,
+					      G_CALLBACK (gcm_calibrate_argyll_exit_cb),
+					      calibrate_argyll);
+	g_signal_handlers_disconnect_by_func (calibrate_argyll->priv->terminal,
+					      G_CALLBACK (gcm_calibrate_argyll_cursor_moved_cb),
+					      calibrate_argyll);
+#endif
 
 	/* hide */
 	gcm_calibrate_dialog_hide (priv->calibrate_dialog);
 
+	if (priv->keypress_id != 0)
+		g_source_remove (priv->keypress_id);
+
 	g_main_loop_unref (priv->loop);
 	g_object_unref (priv->screen);
 	g_object_unref (priv->calibrate_dialog);
@@ -2773,11 +2997,11 @@ gcm_calibrate_argyll_finalize (GObject *object)
  *
  * Return value: a new GcmCalibrateArgyll object.
  **/
-GcmCalibrateArgyll *
+GcmCalibrate *
 gcm_calibrate_argyll_new (void)
 {
 	GcmCalibrateArgyll *calibrate_argyll;
 	calibrate_argyll = g_object_new (GCM_TYPE_CALIBRATE_ARGYLL, NULL);
-	return GCM_CALIBRATE_ARGYLL (calibrate_argyll);
+	return GCM_CALIBRATE (calibrate_argyll);
 }
 
