@@ -37,7 +37,7 @@
 #include "gcm-device-xrandr.h"
 #include "gcm-utils.h"
 #include "gcm-brightness.h"
-#include "gcm-colorimeter.h"
+#include "gcm-sensor-client.h"
 #include "gcm-calibrate-dialog.h"
 #include "gcm-exif.h"
 
@@ -55,12 +55,12 @@ static void     gcm_calibrate_finalize	(GObject     *object);
 struct _GcmCalibratePrivate
 {
 	GcmDmi				*dmi;
-	GcmColorimeter			*colorimeter;
+	GcmSensorClient			*sensor_client;
 	GcmCalibrateReferenceKind	 reference_kind;
 	GcmCalibrateDeviceKind		 calibrate_device_kind;
 	GcmCalibratePrintKind		 print_kind;
 	GcmCalibratePrecision		 precision;
-	GcmColorimeterKind		 colorimeter_kind;
+	GcmSensorKind		 sensor_kind;
 	GcmCalibrateDialog		*calibrate_dialog;
 	GcmDeviceKind			 device_kind;
 	GcmXyz				*xyz;
@@ -90,7 +90,7 @@ enum {
 	PROP_CALIBRATE_DEVICE_KIND,
 	PROP_PRINT_KIND,
 	PROP_DEVICE_KIND,
-	PROP_COLORIMETER_KIND,
+	PROP_SENSOR_KIND,
 	PROP_OUTPUT_NAME,
 	PROP_FILENAME_SOURCE,
 	PROP_FILENAME_REFERENCE,
@@ -104,21 +104,26 @@ enum {
 G_DEFINE_TYPE (GcmCalibrate, gcm_calibrate, G_TYPE_OBJECT)
 
 /**
- * gcm_calibrate_precision_from_string:
+ * gcm_calibrate_mkdir_with_parents:
  **/
-static GcmCalibratePrecision
-gcm_calibrate_precision_from_string (const gchar *string)
+static gboolean
+gcm_calibrate_mkdir_with_parents (const gchar *filename, GError **error)
 {
-	if (g_strcmp0 (string, "short") == 0)
-		return GCM_CALIBRATE_PRECISION_SHORT;
-	if (g_strcmp0 (string, "normal") == 0)
-		return GCM_CALIBRATE_PRECISION_NORMAL;
-	if (g_strcmp0 (string, "long") == 0)
-		return GCM_CALIBRATE_PRECISION_LONG;
-	if (g_strcmp0 (string, "ask") == 0)
-		return GCM_CALIBRATE_PRECISION_UNKNOWN;
-	egg_warning ("failed to convert to precision: %s", string);
-	return GCM_CALIBRATE_PRECISION_UNKNOWN;
+	gboolean ret;
+	GFile *file = NULL;
+
+	/* ensure desination exists */
+	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
+	if (!ret) {
+		file = g_file_new_for_path (filename);
+		ret = g_file_make_directory_with_parents (file, NULL, error);
+		if (!ret)
+			goto out;
+	}
+out:
+	if (file != NULL)
+		g_object_unref (file);
+	return ret;
 }
 
 /**
@@ -248,9 +253,8 @@ gcm_calibrate_set_basename (GcmCalibrate *calibrate)
 		g_string_append_printf (basename, " - %s", serial);
 	g_string_append_printf (basename, " (%04i-%02i-%02i)", date->year, date->month, date->day);
 
-	/* maybe configure in GSettings? */
-	if (0)
-		g_string_append_printf (basename, " [%s]", timespec);
+	/* Use time as we can calibrate more than once per day */
+	g_string_append_printf (basename, " [%s]", timespec);
 
 	/* save this */
 	g_object_set (calibrate, "basename", basename->str, NULL);
@@ -413,7 +417,7 @@ gcm_calibrate_get_display_kind (GcmCalibrate *calibrate, GtkWindow *window, GErr
 
 	/* can this device support projectors? */
 	if (priv->calibrate_device_kind == GCM_CALIBRATE_DEVICE_KIND_PROJECTOR &&
-	    !gcm_colorimeter_supports_projector (priv->colorimeter)) {
+	    !gcm_sensor_supports_projector (gcm_sensor_client_get_sensor (priv->sensor_client))) {
 		/* TRANSLATORS: title, the hardware calibration device does not support projectors */
 		title = _("Could not calibrate and profile using this color measuring instrument");
 
@@ -457,7 +461,7 @@ gcm_calibrate_set_working_path (GcmCalibrate *calibrate, GError **error)
 	timespec = gcm_calibrate_get_time ();
 	folder = g_strjoin (" - ", priv->basename, timespec, NULL);
 	priv->working_path = g_build_filename (g_get_user_config_dir (), "gnome-color-manager", "calibration", folder, NULL);
-	ret = gcm_utils_mkdir_with_parents (priv->working_path, error);
+	ret = gcm_calibrate_mkdir_with_parents (priv->working_path, error);
 	g_free (timespec);
 	g_free (folder);
 	return ret;
@@ -589,7 +593,7 @@ gcm_calibrate_display (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	}
 
 	/* get calibration device model */
-	hardware_device = gcm_colorimeter_get_model (priv->colorimeter);
+	hardware_device = gcm_sensor_get_model (gcm_sensor_client_get_sensor (priv->sensor_client));
 
 	/* get device, harder */
 	if (hardware_device == NULL) {
@@ -610,8 +614,7 @@ gcm_calibrate_display (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	}
 
 	/* get default precision */
-	precision = g_settings_get_string (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -725,8 +728,10 @@ gcm_calibrate_device_get_reference_image (const gchar *directory, GtkWindow *win
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
 
-	/* TRANSLATORS: dialog for file->open dialog */
-	dialog = gtk_file_chooser_dialog_new (_("Select reference image"), window,
+	/* TRANSLATORS: dialog for file->open dialog. A calibration target image is the
+	 * aquired image of the calibration target, e.g. an image file that looks
+	 * a bit like this: http://www.colorreference.de/targets/target.jpg */
+	dialog = gtk_file_chooser_dialog_new (_("Select calibration target image"), window,
 					       GTK_FILE_CHOOSER_ACTION_OPEN,
 					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
@@ -905,8 +910,10 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	/* TRANSLATORS: title, you can profile all at once, or in steps */
 	title = _("Please choose a profiling mode");
 
-	/* TRANSLATORS: dialog message */
-	message = _("Please indicate if you want to profile a local printer, generate some reference images, or process some reference images.");
+	/* TRANSLATORS: dialog message. Test patches are pages of colored squares
+	 * that are printed with a printer, and then read in with a calibration
+	 * device to create a profile */
+	message = _("Please indicate if you want to profile a local printer, generate some test patches, or profile using existing test patches.");
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_PRINT_MODE, title, message);
@@ -924,8 +931,7 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	}
 
 	/* get default precision */
-	precision = g_settings_get_string (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -1002,7 +1008,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	GtkResponseType response;
 	GtkWindow *window_tmp;
 	gchar *precision = NULL;
-#ifdef GCM_USE_PACKAGEKIT
+#ifdef HAVE_PACKAGEKIT
 	GtkWidget *dialog;
 #endif
 	const gchar *title;
@@ -1014,7 +1020,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	/* install shared-color-targets package */
 	has_shared_targets = g_file_test ("/usr/share/shared-color-targets", G_FILE_TEST_IS_DIR);
 	if (!has_shared_targets) {
-#ifdef GCM_USE_PACKAGEKIT
+#ifdef HAVE_PACKAGEKIT
 		/* ask the user to confirm */
 		dialog = gtk_message_dialog_new (window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
 						 /* TRANSLATORS: title, usually we can tell based on the EDID data or output name */
@@ -1046,13 +1052,15 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 #endif
 	}
 
-	/* TRANSLATORS: this is the window title for when the user selects the chart type.
-	                A chart is a type of reference image the user has purchased. */
-	title = _("Please select chart type");
+	/* TRANSLATORS: this is the window title for when the user selects the calibration target.
+	 * A calibration target is an accuratly printed grid of colors, for instance:
+	 * the IT 8.7 targets available here: http://www.targets.coloraid.de/ */
+	title = _("Please select a calibration target");
 	g_string_set_size (string, 0);
 
-	/* TRANSLATORS: dialog message, preface */
-	g_string_append_printf (string, "%s\n", _("Before profiling the device, you have to manually capture an image of a calibrated target and save it as a TIFF image file."));
+	/* TRANSLATORS: dialog message, preface. A calibration target looks like
+	 * this: http://www.colorreference.de/targets/target.jpg */
+	g_string_append_printf (string, "%s\n", _("Before profiling the device, you have to manually capture an image of a calibration target and save it as a TIFF image file."));
 
 	/* scanner specific options */
 	if (priv->device_kind == GCM_DEVICE_KIND_SCANNER) {
@@ -1073,7 +1081,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_string_append_printf (string, "\n%s\n", _("For best results, the reference target should also be less than two years old."));
 
 	/* TRANSLATORS: this is the message body for the chart selection */
-	g_string_append_printf (string, "\n%s\n", _("Please select the chart type which corresponds to your reference file."));
+	g_string_append_printf (string, "\n%s\n", _("Please select the calibration target type which corresponds to your reference file."));
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
@@ -1086,7 +1094,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
 				     GCM_CALIBRATE_ERROR_USER_ABORT,
-				     "user did not choose chart type");
+				     "user did not choose calibration target type");
 		ret = FALSE;
 		goto out;
 	}
@@ -1095,8 +1103,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_object_get (priv->calibrate_dialog, "reference-kind", &priv->reference_kind, NULL);
 
 	/* get default precision */
-	precision = g_settings_get_string (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -1113,7 +1120,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
 				     GCM_CALIBRATE_ERROR_USER_ABORT,
-				     "could not get reference image");
+				     "could not get calibration target image");
 		ret = FALSE;
 		goto out;
 	}
@@ -1195,8 +1202,8 @@ gcm_calibrate_get_property (GObject *object, guint prop_id, GValue *value, GPara
 	case PROP_CALIBRATE_DEVICE_KIND:
 		g_value_set_uint (value, priv->calibrate_device_kind);
 		break;
-	case PROP_COLORIMETER_KIND:
-		g_value_set_uint (value, priv->colorimeter_kind);
+	case PROP_SENSOR_KIND:
+		g_value_set_uint (value, priv->sensor_kind);
 		break;
 	case PROP_OUTPUT_NAME:
 		g_value_set_string (value, priv->output_name);
@@ -1259,13 +1266,17 @@ gcm_calibrate_guess_kind (GcmCalibrate *calibrate)
 }
 
 /**
- * gcm_prefs_colorimeter_changed_cb:
+ * gcm_prefs_sensor_client_changed_cb:
  **/
 static void
-gcm_prefs_colorimeter_changed_cb (GcmColorimeter *_colorimeter, GcmCalibrate *calibrate)
+gcm_prefs_sensor_client_changed_cb (GcmSensorClient *_sensor_client, GcmCalibrate *calibrate)
 {
-	calibrate->priv->colorimeter_kind = gcm_colorimeter_get_kind (_colorimeter);
-	g_object_notify (G_OBJECT (calibrate), "colorimeter-kind");
+	GcmSensor *sensor;
+	calibrate->priv->sensor_kind = GCM_SENSOR_KIND_UNKNOWN;
+	sensor = gcm_sensor_client_get_sensor (_sensor_client);
+	if (sensor != NULL)
+		calibrate->priv->sensor_kind = gcm_sensor_get_kind (sensor);
+	g_object_notify (G_OBJECT (calibrate), "sensor_client-kind");
 }
 
 /**
@@ -1383,12 +1394,12 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 	g_object_class_install_property (object_class, PROP_DEVICE_KIND, pspec);
 
 	/**
-	 * GcmCalibrate:colorimeter-kind:
+	 * GcmCalibrate:sensor-kind:
 	 */
-	pspec = g_param_spec_uint ("colorimeter-kind", NULL, NULL,
+	pspec = g_param_spec_uint ("sensor-kind", NULL, NULL,
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READABLE);
-	g_object_class_install_property (object_class, PROP_COLORIMETER_KIND, pspec);
+	g_object_class_install_property (object_class, PROP_SENSOR_KIND, pspec);
 
 	/**
 	 * GcmCalibrate:output-name:
@@ -1503,6 +1514,8 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 static void
 gcm_calibrate_init (GcmCalibrate *calibrate)
 {
+	GcmSensor *sensor;
+
 	calibrate->priv = GCM_CALIBRATE_GET_PRIVATE (calibrate);
 	calibrate->priv->output_name = NULL;
 	calibrate->priv->filename_source = NULL;
@@ -1520,7 +1533,7 @@ gcm_calibrate_init (GcmCalibrate *calibrate)
 	calibrate->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_UNKNOWN;
 	calibrate->priv->reference_kind = GCM_CALIBRATE_REFERENCE_KIND_UNKNOWN;
 	calibrate->priv->precision = GCM_CALIBRATE_PRECISION_UNKNOWN;
-	calibrate->priv->colorimeter = gcm_colorimeter_new ();
+	calibrate->priv->sensor_client = gcm_sensor_client_new ();
 	calibrate->priv->dmi = gcm_dmi_new ();
 	calibrate->priv->calibrate_dialog = gcm_calibrate_dialog_new ();
 
@@ -1531,8 +1544,12 @@ gcm_calibrate_init (GcmCalibrate *calibrate)
 	calibrate->priv->settings = g_settings_new (GCM_SETTINGS_SCHEMA);
 
 	/* coldplug, and watch for changes */
-	calibrate->priv->colorimeter_kind = gcm_colorimeter_get_kind (calibrate->priv->colorimeter);
-	g_signal_connect (calibrate->priv->colorimeter, "changed", G_CALLBACK (gcm_prefs_colorimeter_changed_cb), calibrate);
+	calibrate->priv->sensor_kind = GCM_SENSOR_KIND_UNKNOWN;
+	sensor = gcm_sensor_client_get_sensor (calibrate->priv->sensor_client);
+	if (sensor != NULL)
+		calibrate->priv->sensor_kind = gcm_sensor_get_kind (sensor);
+
+	g_signal_connect (calibrate->priv->sensor_client, "changed", G_CALLBACK (gcm_prefs_sensor_client_changed_cb), calibrate);
 }
 
 /**
@@ -1555,10 +1572,10 @@ gcm_calibrate_finalize (GObject *object)
 	g_free (priv->device);
 	g_free (priv->serial);
 	g_free (priv->working_path);
-	g_signal_handlers_disconnect_by_func (calibrate->priv->colorimeter, G_CALLBACK (gcm_prefs_colorimeter_changed_cb), calibrate);
+	g_signal_handlers_disconnect_by_func (calibrate->priv->sensor_client, G_CALLBACK (gcm_prefs_sensor_client_changed_cb), calibrate);
 	if (priv->xyz != NULL)
 		g_object_unref (priv->xyz);
-	g_object_unref (priv->colorimeter);
+	g_object_unref (priv->sensor_client);
 	g_object_unref (priv->dmi);
 	g_object_unref (priv->calibrate_dialog);
 	g_object_unref (priv->settings);
