@@ -32,22 +32,22 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gudev/gudev.h>
-#include <libgnomeui/gnome-rr.h>
 #include <cups/cups.h>
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
  #include <sane/sane.h>
 #endif
 
+#include "gcm-x11-screen.h"
 #include "gcm-client.h"
 #include "gcm-device-xrandr.h"
 #include "gcm-device-udev.h"
 #include "gcm-device-cups.h"
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
  #include "gcm-device-sane.h"
 #endif
 #include "gcm-device-virtual.h"
-#include "gcm-screen.h"
+#include "gcm-x11-screen.h"
 #include "gcm-utils.h"
 
 #include "egg-debug.h"
@@ -56,9 +56,9 @@ static void     gcm_client_finalize	(GObject     *object);
 
 #define GCM_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_CLIENT, GcmClientPrivate))
 
-static void gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output);
+static void gcm_client_xrandr_add (GcmClient *client, GcmX11Output *output);
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 static gboolean gcm_client_coldplug_devices_sane (GcmClient *client, GError **error);
 static gpointer gcm_client_coldplug_devices_sane_thrd (GcmClient *client);
 #endif
@@ -74,7 +74,7 @@ struct _GcmClientPrivate
 	GPtrArray			*array;
 	GUdevClient			*gudev_client;
 	GSettings			*settings;
-	GcmScreen			*screen;
+	GcmX11Screen			*screen;
 	http_t				*http;
 	gboolean			 loading;
 	guint				 loading_refcount;
@@ -82,6 +82,7 @@ struct _GcmClientPrivate
 	gboolean			 init_cups;
 	gboolean			 init_sane;
 	guint				 refresh_id;
+	guint				 emit_added_id;
 };
 
 enum {
@@ -263,10 +264,14 @@ gcm_client_remove_device_internal (GcmClient *client, GcmDevice *device, gboolea
 {
 	gboolean ret = FALSE;
 	const gchar *device_id;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 
+	/* lock */
+	g_static_mutex_lock (&mutex);
+
 	/* check device is not connected */
 	device_id = gcm_device_get_id (device);
 	if (gcm_device_get_connected (device)) {
@@ -290,6 +295,9 @@ gcm_client_remove_device_internal (GcmClient *client, GcmDevice *device, gboolea
 		g_signal_emit (client, signals[SIGNAL_REMOVED], 0, device);
 	}
 out:
+	/* unlock */
+	g_static_mutex_unlock (&mutex);
+
 	return ret;
 }
 
@@ -374,7 +382,7 @@ out:
 	return ret;
 }
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 /**
  * gcm_client_sane_refresh_cb:
  **/
@@ -418,7 +426,7 @@ static void
 gcm_client_uevent_cb (GUdevClient *gudev_client, const gchar *action, GUdevDevice *udev_device, GcmClient *client)
 {
 	gboolean ret;
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 	const gchar *value;
 	GcmDevice *device_tmp;
 	guint i;
@@ -431,7 +439,7 @@ gcm_client_uevent_cb (GUdevClient *gudev_client, const gchar *action, GUdevDevic
 		if (ret)
 			egg_debug ("removed %s", g_udev_device_get_sysfs_path (udev_device));
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 		/* we need to remove scanner devices */
 		value = g_udev_device_get_property (udev_device, "GCM_RESCAN");
 		if (g_strcmp0 (value, "scanner") == 0) {
@@ -465,7 +473,7 @@ gcm_client_uevent_cb (GUdevClient *gudev_client, const gchar *action, GUdevDevic
 		if (ret)
 			egg_debug ("added %s", g_udev_device_get_sysfs_path (udev_device));
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 		/* we need to rescan scanner devices */
 		value = g_udev_device_get_property (udev_device, "GCM_RESCAN");
 		if (g_strcmp0 (value, "scanner") == 0) {
@@ -582,41 +590,36 @@ gcm_client_get_device_by_window (GcmClient *client, GdkWindow *window)
 	gfloat covered_max = 0.0f;
 	gint window_width, window_height;
 	gint window_x, window_y;
-	gint x, y;
+	guint x, y;
 	guint i;
-	guint len = 0;
 	guint width, height;
-	GnomeRRMode *mode;
-	GnomeRROutput *output_best = NULL;
-	GnomeRROutput **outputs;
+	GcmX11Output *output;
+	GcmX11Output *output_best = NULL;
+	GPtrArray *outputs;
 	GcmDevice *device = NULL;
 
 	/* get the window parameters, in root co-ordinates */
 	gdk_window_get_origin (window, &window_x, &window_y);
-	gdk_drawable_get_size (GDK_DRAWABLE(window), &window_width, &window_height);
+	window_width = gdk_window_get_width (window);
+	window_height = gdk_window_get_height (window);
 
 	/* get list of updates */
-	outputs = gcm_screen_get_outputs (client->priv->screen, NULL);
+	outputs = gcm_x11_screen_get_outputs (client->priv->screen, NULL);
 	if (outputs == NULL)
 		goto out;
 
-	/* find length */
-	for (i=0; outputs[i] != NULL; i++)
-		len++;
-
 	/* go through each option */
-	for (i=0; i<len; i++) {
+	for (i=0; i<outputs->len; i++) {
 
 		/* not interesting */
-		if (!gnome_rr_output_is_connected (outputs[i]))
+		output = g_ptr_array_index (outputs, i);
+		if (!gcm_x11_output_get_connected (output))
 			continue;
 
 		/* get details about the output */
-		gnome_rr_output_get_position (outputs[i], &x, &y);
-		mode = gnome_rr_output_get_current_mode (outputs[i]);
-		width = gnome_rr_mode_get_width (mode);
-		height = gnome_rr_mode_get_height (mode);
-		egg_debug ("%s: %ix%i -> %ix%i (%ix%i -> %ix%i)", gnome_rr_output_get_name (outputs[i]),
+		gcm_x11_output_get_position (output, &x, &y);
+		gcm_x11_output_get_size (output, &width, &height);
+		egg_debug ("%s: %ix%i -> %ix%i (%ix%i -> %ix%i)", gcm_x11_output_get_name (output),
 			   x, y, x+width, y+height,
 			   window_x, window_y, window_x+window_width, window_y+window_height);
 
@@ -626,7 +629,7 @@ gcm_client_get_device_by_window (GcmClient *client, GdkWindow *window)
 
 		/* keep a running total of which one is best */
 		if (covered > 0.01f && covered > covered_max) {
-			output_best = outputs[i];
+			output_best = output;
 
 			/* optimize */
 			if (covered > 0.99) {
@@ -636,10 +639,12 @@ gcm_client_get_device_by_window (GcmClient *client, GdkWindow *window)
 
 			/* keep looking */
 			covered_max = covered;
-			egg_debug ("personal best of %f for %s", covered, gnome_rr_output_get_name (output_best));
+			egg_debug ("personal best of %f for %s", covered, gcm_x11_output_get_name (output_best));
 		}
 	}
 out:
+	if (outputs != NULL)
+		g_ptr_array_unref (outputs);
 	/* if we found an output, get the device */
 	if (output_best != NULL) {
 		GcmDevice *device_tmp;
@@ -655,16 +660,16 @@ out:
  * gcm_client_xrandr_add:
  **/
 static void
-gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output)
+gcm_client_xrandr_add (GcmClient *client, GcmX11Output *output)
 {
 	gboolean ret;
 	GError *error = NULL;
 	GcmDevice *device = NULL;
 
 	/* if nothing connected then ignore */
-	ret = gnome_rr_output_is_connected (output);
+	ret = gcm_x11_output_get_connected (output);
 	if (!ret) {
-		egg_debug ("%s is not connected", gnome_rr_output_get_name (output));
+		egg_debug ("%s is not connected", gcm_x11_output_get_name (output));
 		goto out;
 	}
 
@@ -695,20 +700,32 @@ out:
 static gboolean
 gcm_client_coldplug_devices_xrandr (GcmClient *client, GError **error)
 {
-	GnomeRROutput **outputs;
+	GcmX11Output *output;
+	GPtrArray *outputs = NULL;
 	guint i;
+	gboolean ret;
 	GcmClientPrivate *priv = client->priv;
 
-	outputs = gcm_screen_get_outputs (priv->screen, error);
-	if (outputs == NULL)
-		return FALSE;
-	for (i=0; outputs[i] != NULL; i++)
-		gcm_client_xrandr_add (client, outputs[i]);
+	/* use the default screen */
+	ret = gcm_x11_screen_assign (priv->screen, NULL, error);
+	if (!ret)
+		goto out;
+	outputs = gcm_x11_screen_get_outputs (priv->screen, error);
+	if (outputs == NULL) {
+		ret = FALSE;
+		goto out;
+	}
 
-	/* inform the UI */
+	/* add each device */
+	for (i=0; i<outputs->len; i++) {
+		output = g_ptr_array_index (outputs, i);
+		gcm_client_xrandr_add (client, output);
+	}
+out:
+	if (outputs != NULL)
+		g_ptr_array_unref (outputs);
 	gcm_client_done_loading (client);
-
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -785,7 +802,7 @@ gcm_client_coldplug_devices_cups_thrd (GcmClient *client)
 	return NULL;
 }
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 /**
  * gcm_client_sane_add:
  **/
@@ -920,7 +937,7 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 	} else if (kind == GCM_DEVICE_KIND_CAMERA) {
 		/* FIXME: use GPhoto? */
 		device = gcm_device_udev_new ();
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 	} else if (kind == GCM_DEVICE_KIND_SCANNER) {
 		device = gcm_device_sane_new ();
 #endif
@@ -1156,7 +1173,7 @@ gcm_client_coldplug (GcmClient *client, GcmClientColdplug coldplug, GError **err
 		}
 	}
 
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 	/* SANE */
 	enable = g_settings_get_boolean (client->priv->settings, GCM_SETTINGS_ENABLE_SANE);
 	if (enable && (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_SANE)) {
@@ -1177,6 +1194,44 @@ out:
 	return ret;
 }
 
+typedef struct {
+	GcmClient *client;
+	GcmDevice *device;
+} GcmClientEmitHelper;
+
+/**
+ * gcm_client_emit_added_idle_cb:
+ **/
+static gboolean
+gcm_client_emit_added_idle_cb (GcmClientEmitHelper *helper)
+{
+	/* emit a signal */
+	egg_debug ("emit added: %s", gcm_device_get_id (helper->device));
+	g_signal_emit (helper->client, signals[SIGNAL_ADDED], 0, helper->device);
+	helper->client->priv->emit_added_id = 0;
+
+	g_object_unref (helper->client);
+	g_object_unref (helper->device);
+	g_free (helper);
+	return FALSE;
+}
+
+/**
+ * gcm_client_emit_added_idle:
+ **/
+static void
+gcm_client_emit_added_idle (GcmClient *client, GcmDevice *device)
+{
+	GcmClientEmitHelper *helper;
+	helper = g_new0 (GcmClientEmitHelper, 1);
+	helper->client = g_object_ref (client);
+	helper->device = g_object_ref (device);
+	client->priv->emit_added_id = g_idle_add ((GSourceFunc) gcm_client_emit_added_idle_cb, helper);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (client->priv->emit_added_id, "[GcmClient] emit added for device");
+#endif
+}
+
 /**
  * gcm_client_add_device:
  **/
@@ -1187,18 +1242,24 @@ gcm_client_add_device (GcmClient *client, GcmDevice *device, GError **error)
 	const gchar *device_id;
 	GcmDevice *device_tmp = NULL;
 	GPtrArray *array;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 
+	/* lock */
+	g_static_mutex_lock (&mutex);
+
 	/* look to see if device already exists */
 	device_id = gcm_device_get_id (device);
 	device_tmp = gcm_client_get_device_by_id (client, device_id);
 	if (device_tmp != NULL) {
 		egg_debug ("already exists, copy settings and remove old device: %s", device_id);
 		array = gcm_device_get_profiles (device_tmp);
-		if (array != NULL)
+		if (array != NULL) {
 			gcm_device_set_profiles (device, array);
+			g_ptr_array_unref (array);
+		}
 		gcm_device_set_saved (device, gcm_device_get_saved (device_tmp));
 		ret = gcm_client_remove_device_internal (client, device_tmp, FALSE, error);
 		if (!ret)
@@ -1212,17 +1273,17 @@ gcm_client_add_device (GcmClient *client, GcmDevice *device, GError **error)
 
 	/* add to the array */
 	g_ptr_array_add (client->priv->array, g_object_ref (device));
-
-	/* emit a signal */
-	egg_debug ("emit added: %s", device_id);
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
 	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 
+	/* emit a signal from the main thread, not this one */
+	gcm_client_emit_added_idle (client, device);
+
 	/* all okay */
 	ret = TRUE;
 out:
+	/* unlock */
+	g_static_mutex_unlock (&mutex);
+
 	if (device_tmp != NULL)
 		g_object_unref (device_tmp);
 	return ret;
@@ -1351,17 +1412,21 @@ gcm_client_set_property (GObject *object, guint prop_id, const GValue *value, GP
  * gcm_client_randr_event_cb:
  **/
 static void
-gcm_client_randr_event_cb (GcmScreen *screen, GcmClient *client)
+gcm_client_randr_event_cb (GcmX11Screen *screen, GcmClient *client)
 {
-	GnomeRROutput **outputs;
+	GPtrArray *outputs;
+	GcmX11Output *output;
 	guint i;
 
 	egg_debug ("screens may have changed");
 
 	/* replug devices */
-	outputs = gcm_screen_get_outputs (screen, NULL);
-	for (i=0; outputs[i] != NULL; i++)
-		gcm_client_xrandr_add (client, outputs[i]);
+	outputs = gcm_x11_screen_get_outputs (screen, NULL);
+	for (i=0; i<outputs->len; i++) {
+		output = g_ptr_array_index (outputs, i);
+		gcm_client_xrandr_add (client, output);
+	}
+	g_ptr_array_unref (outputs);
 }
 
 /**
@@ -1449,8 +1514,8 @@ gcm_client_init (GcmClient *client)
 	client->priv->init_sane = FALSE;
 	client->priv->settings = g_settings_new (GCM_SETTINGS_SCHEMA);
 	client->priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
-	client->priv->screen = gcm_screen_new ();
-	g_signal_connect (client->priv->screen, "outputs-changed",
+	client->priv->screen = gcm_x11_screen_new ();
+	g_signal_connect (client->priv->screen, "changed",
 			  G_CALLBACK (gcm_client_randr_event_cb), client);
 
 	/* use GUdev to find devices */
@@ -1473,6 +1538,8 @@ gcm_client_finalize (GObject *object)
 	/* disconnect anything that's about to fire */
 	if (priv->refresh_id != 0)
 		g_source_remove (priv->refresh_id);
+	if (priv->emit_added_id != 0)
+		g_source_remove (priv->emit_added_id);
 
 	/* do not respond to changed events */
 	for (i=0; i<priv->array->len; i++) {
@@ -1487,7 +1554,7 @@ gcm_client_finalize (GObject *object)
 	g_object_unref (priv->settings);
 	if (client->priv->init_cups)
 		httpClose (priv->http);
-#ifdef GCM_USE_SANE
+#ifdef HAVE_SANE
 	if (client->priv->init_sane)
 		sane_exit ();
 #endif
