@@ -29,7 +29,6 @@
 #include "config.h"
 
 #include <glib-object.h>
-#include <gconf/gconf-client.h>
 
 #include "gcm-device.h"
 #include "gcm-profile.h"
@@ -59,9 +58,8 @@ struct _GcmDevicePrivate
 	gchar			*serial;
 	gchar			*manufacturer;
 	gchar			*model;
-	gchar			*profile_filename;
+	GPtrArray		*profiles;
 	gchar			*title;
-	GConfClient		*gconf_client;
 	GcmColorspace		 colorspace;
 	guint			 changed_id;
 	glong			 modified_time;
@@ -80,7 +78,7 @@ enum {
 	PROP_GAMMA,
 	PROP_BRIGHTNESS,
 	PROP_CONTRAST,
-	PROP_PROFILE_FILENAME,
+	PROP_PROFILES,
 	PROP_TITLE,
 	PROP_COLORSPACE,
 	PROP_LAST
@@ -131,6 +129,9 @@ gcm_device_changed (GcmDevice *device)
 						       (GSourceFunc) gcm_device_changed_cb,
 						       g_object_ref (device),
 						       (GDestroyNotify) g_object_unref);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (device->priv->changed_id, "[GcmDevice] device changed");
+#endif
 out:
 	/* unlock */
 	g_static_mutex_unlock (&mutex);
@@ -171,31 +172,29 @@ gcm_device_kind_to_string (GcmDeviceKind kind)
 }
 
 /**
- * gcm_device_load_from_profile:
+ * gcm_device_load_from_default_profile:
  **/
 static gboolean
-gcm_device_load_from_profile (GcmDevice *device, GError **error)
+gcm_device_load_from_default_profile (GcmDevice *device, GError **error)
 {
 	gboolean ret = TRUE;
+	GcmProfile *profile;
 	GcmDevicePrivate *priv = device->priv;
 
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 
 	/* no profile to load */
-	if (priv->profile_filename == NULL)
+	if (priv->profiles->len == 0)
 		goto out;
 
-	/* load the profile if it's set */
-	if (priv->profile_filename != NULL) {
-
-		/* if the profile was deleted */
-		ret = g_file_test (priv->profile_filename, G_FILE_TEST_EXISTS);
-		if (!ret) {
-			egg_warning ("the file was deleted and can't be loaded: %s", priv->profile_filename);
-			/* this is not fatal */
-			ret = TRUE;
-			goto out;
-		}
+	/* if the profile was deleted */
+	profile = g_ptr_array_index (priv->profiles, 0);
+	ret = g_file_test (gcm_profile_get_filename (profile), G_FILE_TEST_EXISTS);
+	if (!ret) {
+		egg_warning ("the file was deleted and can't be loaded: %s", gcm_profile_get_filename (profile));
+		/* this is not fatal */
+		ret = TRUE;
+		goto out;
 	}
 out:
 	return ret;
@@ -484,27 +483,95 @@ gcm_device_set_title (GcmDevice *device, const gchar *title)
 }
 
 /**
- * gcm_device_get_profile_filename:
+ * gcm_device_get_profiles:
+ *
+ * Return value: the profiles. Free with g_ptr_array_unref()
  **/
-const gchar *
-gcm_device_get_profile_filename (GcmDevice *device)
+GPtrArray *
+gcm_device_get_profiles (GcmDevice *device)
 {
 	g_return_val_if_fail (GCM_IS_DEVICE (device), NULL);
-	return device->priv->profile_filename;
+	return g_ptr_array_ref (device->priv->profiles);
 }
 
 /**
- * gcm_device_set_profile_filename:
+ * gcm_device_set_profiles:
  **/
 void
-gcm_device_set_profile_filename (GcmDevice *device, const gchar *profile_filename)
+gcm_device_set_profiles (GcmDevice *device, GPtrArray *profiles)
 {
 	g_return_if_fail (GCM_IS_DEVICE (device));
-	g_free (device->priv->profile_filename);
-	device->priv->profile_filename = g_strdup (profile_filename);
+	g_return_if_fail (profiles != NULL);
+
+	g_ptr_array_unref (device->priv->profiles);
+	device->priv->profiles = g_ptr_array_ref (profiles);
 	gcm_device_changed (device);
 }
 
+/**
+ * gcm_device_get_default_profile_filename:
+ **/
+const gchar *
+gcm_device_get_default_profile_filename (GcmDevice *device)
+{
+	GcmProfile *profile;
+	g_return_val_if_fail (GCM_IS_DEVICE (device), NULL);
+	if (device->priv->profiles->len == 0)
+		return NULL;
+	profile = g_ptr_array_index (device->priv->profiles, 0);
+	return gcm_profile_get_filename (profile);
+}
+
+/**
+ * gcm_device_get_default_profile:
+ **/
+GcmProfile *
+gcm_device_get_default_profile (GcmDevice *device)
+{
+	GcmProfile *profile;
+	g_return_val_if_fail (GCM_IS_DEVICE (device), NULL);
+	if (device->priv->profiles->len == 0)
+		return NULL;
+	profile = g_ptr_array_index (device->priv->profiles, 0);
+	return profile;
+}
+
+/**
+ * gcm_device_set_default_profile_filename:
+ **/
+void
+gcm_device_set_default_profile_filename (GcmDevice *device, const gchar *profile_filename)
+{
+	GcmProfile *profile;
+	GPtrArray *array;
+	gboolean ret;
+	GFile *file;
+	GError *error = NULL;
+
+	g_return_if_fail (GCM_IS_DEVICE (device));
+
+	/* create new list */
+	array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	profile = gcm_profile_new ();
+
+	/* TODO: parse here? */
+	file = g_file_new_for_path (profile_filename);
+	ret = gcm_profile_parse (profile, file, &error);
+	if (!ret) {
+		egg_warning ("failed to parse: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* FIXME: don't just nuke the list */
+	g_ptr_array_add (array, g_object_ref (profile));
+	gcm_device_set_profiles (device, array);
+out:
+	g_ptr_array_unref (array);
+	g_object_unref (profile);
+	g_object_unref (file);
+}
+
 /**
  * gcm_device_get_modified_time:
  **/
@@ -527,6 +594,10 @@ gcm_device_load (GcmDevice *device, GError **error)
 	gchar *filename = NULL;
 	GTimeVal timeval;
 	gchar *iso_date = NULL;
+	gchar **profile_filenames = NULL;
+	guint i;
+	GcmProfile *profile;
+	GFile *file_tmp;
 	GcmDevicePrivate *priv = device->priv;
 
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
@@ -567,8 +638,26 @@ gcm_device_load (GcmDevice *device, GError **error)
 	gcm_device_set_saved (device, TRUE);
 
 	/* load data */
-	g_free (priv->profile_filename);
-	priv->profile_filename = g_key_file_get_string (file, priv->id, "profile", NULL);
+	g_ptr_array_set_size (priv->profiles, 0);
+
+	/* parse filenames to object, skipping entries that fail to parse */
+	profile_filenames = g_key_file_get_string_list (file, priv->id, "profile", NULL, NULL);
+	if (profile_filenames != NULL) {
+		for (i=0; profile_filenames[i] != NULL; i++) {
+			file_tmp = g_file_new_for_path (profile_filenames[i]);
+			profile = gcm_profile_new ();
+			ret = gcm_profile_parse (profile, file_tmp, &error_local);
+			if (ret) {
+				g_ptr_array_add (priv->profiles, g_object_ref (profile));
+			} else {
+				egg_warning ("failed to parse %s: %s", profile_filenames[i], error_local->message);
+				g_clear_error (&error_local);
+			}
+			g_object_unref (profile);
+			g_object_unref (file_tmp);
+		}
+	}
+
 	if (priv->serial == NULL)
 		priv->serial = g_key_file_get_string (file, priv->id, "serial", NULL);
 	if (priv->model == NULL)
@@ -577,9 +666,7 @@ gcm_device_load (GcmDevice *device, GError **error)
 		priv->manufacturer = g_key_file_get_string (file, priv->id, "manufacturer", NULL);
 	priv->gamma = g_key_file_get_double (file, priv->id, "gamma", &error_local);
 	if (error_local != NULL) {
-		priv->gamma = gconf_client_get_float (priv->gconf_client, "/apps/gnome-color-manager/default_gamma", NULL);
-		if (priv->gamma < 0.1f)
-			priv->gamma = 1.0f;
+		priv->gamma = 1;
 		g_clear_error (&error_local);
 	}
 	priv->brightness = g_key_file_get_double (file, priv->id, "brightness", &error_local);
@@ -608,19 +695,19 @@ gcm_device_load (GcmDevice *device, GError **error)
 	priv->modified_time = timeval.tv_sec;
 
 	/* load this */
-	ret = gcm_device_load_from_profile (device, &error_local);
+	ret = gcm_device_load_from_default_profile (device, &error_local);
 	if (!ret) {
 
 		/* just print a warning, this is not fatal */
-		egg_warning ("failed to load profile %s: %s", priv->profile_filename, error_local->message);
+		egg_warning ("failed to load profile %s", error_local->message);
 		g_error_free (error_local);
 
 		/* recover as the file might have been corrupted */
-		g_free (priv->profile_filename);
-		priv->profile_filename = NULL;
+		g_ptr_array_set_size (priv->profiles, 0);
 		ret = TRUE;
 	}
 out:
+	g_strfreev (profile_filenames);
 	g_free (iso_date);
 	g_free (filename);
 	if (file != NULL)
@@ -635,15 +722,21 @@ gboolean
 gcm_device_save (GcmDevice *device, GError **error)
 {
 	GKeyFile *keyfile = NULL;
+	guint i;
 	gboolean ret;
 	gchar *data = NULL;
 	gchar *dirname;
 	GFile *file = NULL;
 	gchar *filename = NULL;
 	gchar *timespec = NULL;
+	gchar **profile_filenames;
+	gchar *config_data = NULL;
+	gchar **config_items = NULL;
 	GError *error_local = NULL;
 	GTimeVal timeval;
+	GcmProfile *profile;
 	GcmDevicePrivate *priv = device->priv;
+	GcmDeviceClass *klass = GCM_DEVICE_GET_CLASS (device);
 
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 	g_return_val_if_fail (priv->id != NULL, FALSE);
@@ -703,10 +796,19 @@ gcm_device_save (GcmDevice *device, GError **error)
 	g_key_file_set_string (keyfile, priv->id, "modified", timespec);
 
 	/* save data */
-	if (priv->profile_filename == NULL)
+	if (priv->profiles->len == 0)
 		g_key_file_remove_key (keyfile, priv->id, "profile", NULL);
-	else
-		g_key_file_set_string (keyfile, priv->id, "profile", priv->profile_filename);
+	else {
+		profile_filenames = g_new0 (gchar *, priv->profiles->len + 1);
+		for (i=0; i<priv->profiles->len; i++) {
+			profile = g_ptr_array_index (priv->profiles, i);
+			profile_filenames[i] = g_strdup (gcm_profile_get_filename (profile));
+		}
+		g_key_file_set_string_list (keyfile, priv->id, "profile",
+					    (const gchar * const*) profile_filenames,
+					    priv->profiles->len);
+		g_strfreev (profile_filenames);
+	}
 
 	/* save device specific data */
 	if (priv->serial == NULL)
@@ -752,6 +854,20 @@ gcm_device_save (GcmDevice *device, GError **error)
 	if (priv->virtual)
 		g_key_file_set_boolean (keyfile, priv->id, "virtual", TRUE);
 
+	/* get extra, device specific config data */
+	if (klass->get_config_data != NULL) {
+		config_data = klass->get_config_data (device);
+		config_items = g_strsplit (config_data, "=", 2);
+		if (g_strv_length (config_items) != 2) {
+			ret = FALSE;
+			g_set_error (error, 1, 0, "failed to get device specific config data for: %s", priv->id);
+			goto out;
+		}
+
+		/* save unstructured data to the keyfile */
+		g_key_file_set_string (keyfile, priv->id, config_items[0], config_items[1]);
+	}
+
 	/* convert to string */
 	data = g_key_file_to_data (keyfile, NULL, &error_local);
 	if (data == NULL) {
@@ -772,6 +888,8 @@ gcm_device_save (GcmDevice *device, GError **error)
 	/* update status */
 	gcm_device_set_saved (device, TRUE);
 out:
+	g_strfreev (config_items);
+	g_free (config_data);
 	g_free (timespec);
 	g_free (data);
 	g_free (filename);
@@ -847,8 +965,8 @@ gcm_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 	case PROP_CONTRAST:
 		g_value_set_float (value, priv->contrast);
 		break;
-	case PROP_PROFILE_FILENAME:
-		g_value_set_string (value, priv->profile_filename);
+	case PROP_PROFILES:
+		g_value_set_pointer (value, priv->profiles);
 		break;
 	case PROP_TITLE:
 		g_value_set_string (value, priv->title);
@@ -898,8 +1016,8 @@ gcm_device_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	case PROP_TITLE:
 		gcm_device_set_title (device, g_value_get_string (value));
 		break;
-	case PROP_PROFILE_FILENAME:
-		gcm_device_set_profile_filename (device, g_value_get_string (value));
+	case PROP_PROFILES:
+		gcm_device_set_profiles (device, g_value_get_pointer (value));
 		break;
 	case PROP_GAMMA:
 		gcm_device_set_gamma (device, g_value_get_float (value));
@@ -1020,12 +1138,11 @@ gcm_device_class_init (GcmDeviceClass *klass)
 	g_object_class_install_property (object_class, PROP_CONTRAST, pspec);
 
 	/**
-	 * GcmDevice:profile-filename:
+	 * GcmDevice:profiles:
 	 */
-	pspec = g_param_spec_string ("profile-filename", NULL, NULL,
-				     NULL,
-				     G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_PROFILE_FILENAME, pspec);
+	pspec = g_param_spec_pointer ("profiles", NULL, NULL,
+				      G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_PROFILES, pspec);
 
 	/**
 	 * GcmDevice:title:
@@ -1062,7 +1179,6 @@ gcm_device_class_init (GcmDeviceClass *klass)
 static void
 gcm_device_init (GcmDevice *device)
 {
-	GError *error = NULL;
 	device->priv = GCM_DEVICE_GET_PRIVATE (device);
 	device->priv->changed_id = 0;
 	device->priv->id = NULL;
@@ -1072,16 +1188,9 @@ gcm_device_init (GcmDevice *device)
 	device->priv->serial = NULL;
 	device->priv->manufacturer = NULL;
 	device->priv->model = NULL;
-	device->priv->profile_filename = NULL;
+	device->priv->profiles = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	device->priv->modified_time = 0;
-	device->priv->gconf_client = gconf_client_get_default ();
-	device->priv->gamma = gconf_client_get_float (device->priv->gconf_client, GCM_SETTINGS_DEFAULT_GAMMA, &error);
-	if (error != NULL) {
-		egg_warning ("failed to get setup parameters: %s", error->message);
-		g_error_free (error);
-	}
-	if (device->priv->gamma < 0.01)
-		device->priv->gamma = 1.0f;
+	device->priv->gamma = 1.0;
 	device->priv->brightness = 0.0f;
 	device->priv->contrast = 100.f;
 	device->priv->colorspace = GCM_COLORSPACE_UNKNOWN;
@@ -1105,226 +1214,8 @@ gcm_device_finalize (GObject *object)
 	g_free (priv->serial);
 	g_free (priv->manufacturer);
 	g_free (priv->model);
-	g_free (priv->profile_filename);
-	g_object_unref (priv->gconf_client);
+	g_ptr_array_unref (priv->profiles);
 
 	G_OBJECT_CLASS (gcm_device_parent_class)->finalize (object);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-#include "gcm-device-udev.h"
-
-static guint _changes = 0;
-
-static void
-gcm_device_test_changed_cb (GcmDevice *device, EggTest *test)
-{
-	egg_debug ("emit changed: %s", gcm_device_get_id (device));
-	_changes++;
-
-	egg_test_loop_quit (test);
-}
-
-void
-gcm_device_test (EggTest *test)
-{
-	GcmDevice *device;
-	gboolean ret;
-	GError *error = NULL;
-	const gchar *filename;
-	const gchar *profile;
-	gchar *data;
-	const gchar *kind;
-	GcmDeviceKind kind_enum;
-
-	if (!egg_test_start (test, "GcmDevice"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get a device object");
-	device = gcm_device_udev_new ();
-	egg_test_assert (test, device != NULL);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_device_test_changed_cb), test);
-
-	/************************************************************/
-	egg_test_title (test, "correct number of changed signals");
-	if (_changes == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "changes: %i", _changes);
-
-	/************************************************************/
-	egg_test_title (test, "convert to recognized enum");
-	kind_enum = gcm_device_kind_from_string ("scanner");
-	egg_test_assert (test, (kind_enum == GCM_DEVICE_KIND_SCANNER));
-
-	/************************************************************/
-	egg_test_title (test, "convert to unrecognized enum");
-	kind_enum = gcm_device_kind_from_string ("xxx");
-	egg_test_assert (test, (kind_enum == GCM_DEVICE_KIND_UNKNOWN));
-
-	/************************************************************/
-	egg_test_title (test, "convert from recognized enum");
-	kind = gcm_device_kind_to_string (GCM_DEVICE_KIND_SCANNER);
-	if (g_strcmp0 (kind, "scanner") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid value: %s", kind);
-
-	/************************************************************/
-	egg_test_title (test, "convert from unrecognized enum");
-	kind = gcm_device_kind_to_string (GCM_DEVICE_KIND_UNKNOWN);
-	if (g_strcmp0 (kind, "unknown") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid value: %s", kind);
-
-	/* set some properties */
-	g_object_set (device,
-		      "kind", GCM_DEVICE_KIND_SCANNER,
-		      "id", "sysfs_dummy_device",
-		      "connected", FALSE,
-		      "virtual", FALSE,
-		      "serial", "0123456789",
-		      "colorspace", GCM_COLORSPACE_RGB,
-		      NULL);
-
-	/************************************************************/
-	egg_test_loop_wait (test, 100);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "correct number of changed signals");
-	if (_changes == 1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "changes: %i", _changes);
-
-	/************************************************************/
-	gcm_device_set_connected (device, TRUE);
-	egg_test_loop_wait (test, 100);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "correct number of changed signals");
-	if (_changes == 2)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "changes: %i", _changes);
-
-	/************************************************************/
-	egg_test_title (test, "get id");
-	kind = gcm_device_get_id (device);
-	if (g_strcmp0 (kind, "sysfs_dummy_device") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid id: %s", kind);
-
-	/* ensure the file is nuked */
-	filename = gcm_utils_get_default_config_location ();
-	g_unlink (filename);
-
-	/************************************************************/
-	egg_test_title (test, "load from missing file");
-	ret = gcm_device_load (device, &error);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to load: %s", error->message);
-
-	/* get some properties */
-	profile = gcm_device_get_profile_filename (device);
-
-	/************************************************************/
-	egg_test_title (test, "get profile filename");
-	if (g_strcmp0 (profile, NULL) == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid profile: %s", profile);
-
-	/* empty file that exists */
-	g_file_set_contents (filename, "", -1, NULL);
-
-	/************************************************************/
-	egg_test_title (test, "load from empty file");
-	ret = gcm_device_load (device, &error);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to load: %s", error->message);
-
-	/* set default file */
-	g_file_set_contents (filename, "[sysfs_dummy_device]\ntitle=Canon - CanoScan\ntype=scanner\nprofile=/srv/sysfs_canon_canoscan.icc\n", -1, NULL);
-
-	/************************************************************/
-	egg_test_title (test, "load from configured file");
-	ret = gcm_device_load (device, &error);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to load: %s", error->message);
-
-	/* get some properties */
-	profile = gcm_device_get_profile_filename (device);
-
-	/************************************************************/
-	egg_test_loop_wait (test, 100);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "correct number of changed signals");
-	if (_changes == 3)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "changes: %i", _changes);
-
-	/************************************************************/
-	egg_test_title (test, "get profile filename");
-	if (g_strcmp0 (profile, "/srv/sysfs_canon_canoscan.icc") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid profile: %s", profile);
-
-	/* set some properties */
-	gcm_device_set_profile_filename (device, "/srv/sysfs_canon_canoscan.icc");
-
-	/* ensure the file is nuked, again */
-	g_unlink (filename);
-
-	/************************************************************/
-	egg_test_title (test, "save to empty file");
-	ret = gcm_device_save (device, &error);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to load: %s", error->message);
-
-	/************************************************************/
-	egg_test_title (test, "get contents of saved file");
-	ret = g_file_get_contents (filename, &data, NULL, NULL);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "check data");
-	if (g_str_has_suffix (data, "profile=/srv/sysfs_canon_canoscan.icc\nserial=0123456789\ntype=scanner\ncolorspace=rgb\n"))
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "invalid data: %s", data);
-	g_free (data);
-
-	/* ensure the file is nuked, in case we are running in distcheck */
-	g_unlink (filename);
-
-	g_object_unref (device);
-
-	egg_test_end (test);
-}
-#endif
-
