@@ -28,12 +28,12 @@
 #include <gio/gio.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <lcms.h>
+#include <lcms2.h>
 
 #include "egg-debug.h"
 
 #include "gcm-calibrate-argyll.h"
-#include "gcm-colorimeter.h"
+#include "gcm-sensor-client.h"
 #include "gcm-profile-store.h"
 #include "gcm-utils.h"
 #include "gcm-xyz.h"
@@ -45,6 +45,9 @@ static GcmCalibrate *calibrate = NULL;
 static GcmProfileStore *profile_store = NULL;
 static const gchar *profile_filename = NULL;
 static gboolean done_measure = FALSE;
+static GcmSensor *sensor = NULL;
+static GcmColorXYZ last_sample;
+static gdouble last_ambient = -1.0f;
 
 enum {
 	GCM_PREFS_COMBO_COLUMN_TEXT,
@@ -79,6 +82,130 @@ gcm_picker_set_pixbuf_color (GdkPixbuf *pixbuf, gchar red, gchar green, gchar bl
 	}
 }
 
+/**
+ * gcm_picker_refresh_results:
+ **/
+static void
+gcm_picker_refresh_results (void)
+{
+	GtkImage *image;
+	GtkLabel *label;
+	GdkPixbuf *pixbuf = NULL;
+	GcmColorRGBint color_rgb;
+	GcmColorLab color_lab;
+	GcmColorXYZ color_xyz;
+	GcmColorXYZ color_error;
+	gchar *text_xyz = NULL;
+	gchar *text_lab = NULL;
+	gchar *text_rgb = NULL;
+	gchar *text_error = NULL;
+	gchar *text_ambient = NULL;
+	cmsHPROFILE profile_xyz;
+	cmsHPROFILE profile_rgb;
+	cmsHPROFILE profile_lab;
+	cmsHTRANSFORM transform_rgb;
+	cmsHTRANSFORM transform_lab;
+	cmsHTRANSFORM transform_error;
+
+	/* copy as we're modifying the value */
+	gcm_color_copy_XYZ (&last_sample, &color_xyz);
+
+	/* create new pixbuf of the right size */
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 200, 200);
+
+	/* lcms scales these for some reason */
+	color_xyz.X /= 100.0f;
+	color_xyz.Y /= 100.0f;
+	color_xyz.Z /= 100.0f;
+
+	/* get profiles */
+	profile_xyz = cmsCreateXYZProfile ();
+	profile_rgb = cmsOpenProfileFromFile (profile_filename, "r");
+	profile_lab = cmsCreateLab4Profile (cmsD50_xyY ());
+
+	/* create transforms */
+	transform_rgb = cmsCreateTransform (profile_xyz, TYPE_XYZ_DBL, profile_rgb, TYPE_RGB_8, INTENT_PERCEPTUAL, 0);
+	transform_lab = cmsCreateTransform (profile_xyz, TYPE_XYZ_DBL, profile_lab, TYPE_Lab_DBL, INTENT_PERCEPTUAL, 0);
+	transform_error = cmsCreateTransform (profile_rgb, TYPE_RGB_8, profile_xyz, TYPE_XYZ_DBL, INTENT_PERCEPTUAL, 0);
+
+	cmsDoTransform (transform_rgb, &color_xyz, &color_rgb, 1);
+	cmsDoTransform (transform_lab, &color_xyz, &color_lab, 1);
+	cmsDoTransform (transform_error, &color_rgb, &color_error, 1);
+
+	/* destroy lcms state */
+	cmsDeleteTransform (transform_rgb);
+	cmsDeleteTransform (transform_lab);
+	cmsDeleteTransform (transform_error);
+	cmsCloseProfile (profile_xyz);
+	cmsCloseProfile (profile_rgb);
+	cmsCloseProfile (profile_lab);
+
+	/* set XYZ */
+	label = GTK_LABEL (gtk_builder_get_object (builder, "label_xyz"));
+	text_xyz = g_strdup_printf ("%.3f, %.3f, %.3f", last_sample.X, last_sample.Y, last_sample.Z);
+	gtk_label_set_label (label, text_xyz);
+
+	/* set LAB */
+	label = GTK_LABEL (gtk_builder_get_object (builder, "label_lab"));
+	text_lab = g_strdup_printf ("%.3f, %.3f, %.3f", color_lab.L, color_lab.a, color_lab.b);
+	gtk_label_set_label (label, text_lab);
+
+	/* set RGB */
+	label = GTK_LABEL (gtk_builder_get_object (builder, "label_rgb"));
+	text_rgb = g_strdup_printf ("%i, %i, %i (#%02X%02X%02X)",
+				    color_rgb.R, color_rgb.G, color_rgb.B,
+				    color_rgb.R, color_rgb.G, color_rgb.B);
+	gtk_label_set_label (label, text_rgb);
+	gcm_picker_set_pixbuf_color (pixbuf, color_rgb.R, color_rgb.G, color_rgb.B);
+
+	/* set error */
+	label = GTK_LABEL (gtk_builder_get_object (builder, "label_error"));
+	text_error = g_strdup_printf ("%.1f%%, %.1f%%, %.1f%%",
+				      ABS ((color_error.X - color_xyz.X) / color_xyz.X * 100),
+				      ABS ((color_error.Y - color_xyz.Y) / color_xyz.Y * 100),
+				      ABS ((color_error.Z - color_xyz.Z) / color_xyz.Z * 100));
+	gtk_label_set_label (label, text_error);
+
+	/* set ambient */
+	label = GTK_LABEL (gtk_builder_get_object (builder, "label_ambient"));
+	if (last_ambient < 0) {
+		/* TRANSLATORS: this is when the ambient light level is unknown */
+		gtk_label_set_label (label, _("Unknown"));
+	} else {
+		text_ambient = g_strdup_printf ("%.1f Lux", last_ambient);
+		gtk_label_set_label (label, text_ambient);
+	}
+
+	/* set image */
+	image = GTK_IMAGE (gtk_builder_get_object (builder, "image_preview"));
+	gtk_image_set_from_pixbuf (image, pixbuf);
+
+	g_free (text_xyz);
+	g_free (text_lab);
+	g_free (text_rgb);
+	g_free (text_error);
+	g_free (text_ambient);
+	if (pixbuf != NULL)
+		g_object_unref (pixbuf);
+}
+
+/**
+ * gcm_picker_got_results:
+ **/
+static void
+gcm_picker_got_results (void)
+{
+	GtkWidget *widget;
+
+	/* set expanded */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "expander_results"));
+	gtk_expander_set_expanded (GTK_EXPANDER (widget), TRUE);
+	gtk_widget_set_sensitive (widget, TRUE);
+
+	/* we've got results so make sure it's sensitive */
+	done_measure = TRUE;
+}
+
 /**
  * gcm_picker_measure_cb:
  **/
@@ -93,118 +220,40 @@ gcm_picker_measure_cb (GtkWidget *widget, gpointer data)
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "image_preview"));
 	gtk_image_set_from_file (GTK_IMAGE (widget), DATADIR "/icons/hicolor/64x64/apps/gnome-color-manager.png");
 
-	/* get value */
-	window = GTK_WINDOW (gtk_builder_get_object (builder, "dialog_picker"));
-	ret = gcm_calibrate_spotread (calibrate, window, &error);
-	if (!ret) {
-		egg_warning ("failed to get spot color: %s", error->message);
-		g_error_free (error);
+	if (gcm_sensor_is_native (sensor)) {
+
+		/* set mode */
+		gcm_sensor_set_output_type (sensor, GCM_SENSOR_OUTPUT_TYPE_LCD);
+
+		/* get ambient */
+		ret = gcm_sensor_get_ambient (sensor, NULL, &last_ambient, &error);
+		if (!ret) {
+			g_warning ("failed to get ambient: %s", error->message);
+			g_error_free (error);
+			goto out;
+		}
+
+		/* sample color */
+		ret = gcm_sensor_sample (sensor, NULL, &last_sample, &error);
+		if (!ret) {
+			egg_warning ("failed to measure: %s", error->message);
+			g_error_free (error);
+			goto out;
+		}
+		gcm_picker_refresh_results ();
+		gcm_picker_got_results ();
+	} else {
+		/* get value */
+		window = GTK_WINDOW (gtk_builder_get_object (builder, "dialog_picker"));
+		ret = gcm_calibrate_spotread (calibrate, window, &error);
+		if (!ret) {
+			egg_warning ("failed to get spot color: %s", error->message);
+			g_error_free (error);
+			goto out;
+		}
 	}
-}
-
-/**
- * gcm_picker_refresh_results:
- **/
-static void
-gcm_picker_refresh_results (void)
-{
-	GcmXyz *xyz = NULL;
-	GtkImage *image;
-	GtkLabel *label;
-	GdkPixbuf *pixbuf = NULL;
-	gdouble color_xyz[3];
-	guint8 color_rgb[3];
-	gdouble color_lab[3];
-	gdouble color_error[3];
-	gchar *text_xyz = NULL;
-	gchar *text_lab = NULL;
-	gchar *text_rgb = NULL;
-	gchar *text_error = NULL;
-	cmsHPROFILE profile_xyz;
-	cmsHPROFILE profile_rgb;
-	cmsHPROFILE profile_lab;
-	cmsHTRANSFORM transform_rgb;
-	cmsHTRANSFORM transform_lab;
-	cmsHTRANSFORM transform_error;
-
-	/* get new value */
-	g_object_get (calibrate, "xyz", &xyz, NULL);
-
-	/* create new pixbuf of the right size */
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 200, 200);
-
-	/* get values */
-	g_object_get (xyz,
-		      "cie-x", &color_xyz[0],
-		      "cie-y", &color_xyz[1],
-		      "cie-z", &color_xyz[2],
-		      NULL);
-
-	/* lcms scales these for some reason */
-	color_xyz[0] /= 100.0f;
-	color_xyz[1] /= 100.0f;
-	color_xyz[2] /= 100.0f;
-
-	/* get profiles */
-	profile_xyz = cmsCreateXYZProfile ();
-	profile_rgb = cmsOpenProfileFromFile (profile_filename, "r");
-	profile_lab = cmsCreateLabProfile (cmsD50_xyY ());
-
-	/* create transforms */
-	transform_rgb = cmsCreateTransform (profile_xyz, TYPE_XYZ_DBL, profile_rgb, TYPE_RGB_8, INTENT_PERCEPTUAL, 0);
-	transform_lab = cmsCreateTransform (profile_xyz, TYPE_XYZ_DBL, profile_lab, TYPE_Lab_DBL, INTENT_PERCEPTUAL, 0);
-	transform_error = cmsCreateTransform (profile_rgb, TYPE_RGB_8, profile_xyz, TYPE_XYZ_DBL, INTENT_PERCEPTUAL, 0);
-
-	cmsDoTransform (transform_rgb, color_xyz, color_rgb, 1);
-	cmsDoTransform (transform_lab, color_xyz, color_lab, 1);
-	cmsDoTransform (transform_error, color_rgb, color_error, 1);
-
-	/* destroy lcms state */
-	cmsDeleteTransform (transform_rgb);
-	cmsDeleteTransform (transform_lab);
-	cmsDeleteTransform (transform_error);
-	cmsCloseProfile (profile_xyz);
-	cmsCloseProfile (profile_rgb);
-	cmsCloseProfile (profile_lab);
-
-	/* set XYZ */
-	label = GTK_LABEL (gtk_builder_get_object (builder, "label_xyz"));
-	text_xyz = g_strdup_printf ("%.3f, %.3f, %.3f", color_xyz[0], color_xyz[1], color_xyz[2]);
-	gtk_label_set_label (label, text_xyz);
-
-	/* set LAB */
-	label = GTK_LABEL (gtk_builder_get_object (builder, "label_lab"));
-	text_lab = g_strdup_printf ("%.3f, %.3f, %.3f", color_lab[0], color_lab[1], color_lab[2]);
-	gtk_label_set_label (label, text_lab);
-
-	/* set RGB */
-	label = GTK_LABEL (gtk_builder_get_object (builder, "label_rgb"));
-	text_rgb = g_strdup_printf ("%i, %i, %i (#%02X%02X%02X)",
-				    color_rgb[0], color_rgb[1], color_rgb[2],
-				    color_rgb[0], color_rgb[1], color_rgb[2]);
-	gtk_label_set_label (label, text_rgb);
-	gcm_picker_set_pixbuf_color (pixbuf, color_rgb[0], color_rgb[1], color_rgb[2]);
-
-	/* set error */
-	label = GTK_LABEL (gtk_builder_get_object (builder, "label_error"));
-	text_error = g_strdup_printf ("%.1f%%, %.1f%%, %.1f%%",
-				      ABS ((color_error[0] - color_xyz[0]) / color_xyz[0] * 100),
-				      ABS ((color_error[1] - color_xyz[1]) / color_xyz[1] * 100),
-				      ABS ((color_error[2] - color_xyz[2]) / color_xyz[2] * 100));
-	gtk_label_set_label (label, text_error);
-
-	/* set image */
-	image = GTK_IMAGE (gtk_builder_get_object (builder, "image_preview"));
-	gtk_image_set_from_pixbuf (image, pixbuf);
-
-	g_free (text_xyz);
-	g_free (text_lab);
-	g_free (text_rgb);
-	g_free (text_error);
-	if (xyz != NULL)
-		g_object_unref (xyz);
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
+out:
+	return;
 }
 
 /**
@@ -213,17 +262,20 @@ gcm_picker_refresh_results (void)
 static void
 gcm_picker_xyz_notify_cb (GcmCalibrate *calibrate_, GParamSpec *pspec, gpointer user_data)
 {
-	GtkWidget *widget;
+	GcmXyz *xyz;
 
-	/* set expanded */
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "expander_results"));
-	gtk_expander_set_expanded (GTK_EXPANDER (widget), TRUE);
-	gtk_widget_set_sensitive (widget, TRUE);
+	/* get new value */
+	g_object_get (calibrate, "xyz", &xyz, NULL);
 
-	/* we've got results so make sure it's sensitive */
-	done_measure = TRUE;
+	g_object_get (xyz,
+		      "cie-x", &last_sample.X,
+		      "cie-y", &last_sample.Y,
+		      "cie-z", &last_sample.Z,
+		      NULL);
 
 	gcm_picker_refresh_results ();
+	gcm_picker_got_results ();
+	g_object_unref (xyz);
 }
 
 /**
@@ -256,30 +308,43 @@ gcm_picker_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
 }
 
 /**
- * gcm_picker_colorimeter_setup_ui:
+ * gcm_picker_sensor_client_setup_ui:
  **/
 static void
-gcm_picker_colorimeter_setup_ui (GcmColorimeter *colorimeter)
+gcm_picker_sensor_client_setup_ui (GcmSensorClient *sensor_client)
 {
-	gboolean present;
-	gboolean supports_spot;
-	gboolean ret;
+	gboolean ret = FALSE;
 	GtkWidget *widget;
 
-	present = gcm_colorimeter_get_present (colorimeter);
-	supports_spot = gcm_colorimeter_supports_spot (colorimeter);
-	ret = (present && supports_spot);
-
-	/* change the label */
-	if (present && !supports_spot) {
-		/* TRANSLATORS: this is displayed the user has not got suitable hardware */
-		gtk_label_set_label (GTK_LABEL (info_bar_hardware_label), _("The attached colorimeter is not capable of reading a spot color."));
-	} else if (!present) {
+	/* no present */
+	sensor = gcm_sensor_client_get_sensor (sensor_client);
+	if (sensor == NULL) {
 		/* TRANSLATORS: this is displayed the user has not got suitable hardware */
 		gtk_label_set_label (GTK_LABEL (info_bar_hardware_label), _("No colorimeter is attached."));
+		goto out;
 	}
 
-	/* hide some stuff */
+#ifndef HAVE_VTE
+	if (!gcm_sensor_is_native (sensor)) {
+		 /* TRANSLATORS: this is displayed if VTE support is not enabled */
+		gtk_label_set_label (GTK_LABEL (info_bar_hardware_label), _("This application was compiled without VTE support."));
+		goto out;
+	}
+#endif
+
+#if 0
+	/* no support */
+	ret = gcm_sensor_supports_spot (sensor);
+	if (!ret) {
+		/* TRANSLATORS: this is displayed the user has not got suitable hardware */
+		gtk_label_set_label (GTK_LABEL (info_bar_hardware_label), _("The attached colorimeter is not capable of reading a spot color."));
+		goto out;
+	}
+#else
+	ret = TRUE;
+#endif
+
+out:
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_measure"));
 	gtk_widget_set_sensitive (widget, ret);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "expander_results"));
@@ -288,12 +353,12 @@ gcm_picker_colorimeter_setup_ui (GcmColorimeter *colorimeter)
 }
 
 /**
- * gcm_picker_colorimeter_changed_cb:
+ * gcm_picker_sensor_client_changed_cb:
  **/
 static void
-gcm_picker_colorimeter_changed_cb (GcmColorimeter *colorimeter, gpointer user_data)
+gcm_picker_sensor_client_changed_cb (GcmSensorClient *sensor_client, gpointer user_data)
 {
-	gcm_picker_colorimeter_setup_ui (colorimeter);
+	gcm_picker_sensor_client_setup_ui (sensor_client);
 }
 
 /**
@@ -315,17 +380,15 @@ gcm_window_set_parent_xid (GtkWindow *window, guint32 xid)
 	gdk_window_set_transient_for (our_window, parent_window);
 }
 
-/*
- * gcm_picker_lcms_error_cb:
- */
-static gint
-gcm_picker_lcms_error_cb (gint error_code, const gchar *error_text)
+/**
+ * gcm_picker_error_cb:
+ **/
+static void
+gcm_picker_error_cb (cmsContext ContextID, cmsUInt32Number errorcode, const char *text)
 {
-	egg_warning ("LCMS error %i: %s", error_code, error_text);
-	return LCMS_ERRC_WARNING;
+	egg_warning ("LCMS error %i: %s", errorcode, text);
 }
 
-
 /**
  * gcm_prefs_space_combo_changed_cb:
  **/
@@ -352,6 +415,7 @@ gcm_prefs_space_combo_changed_cb (GtkWidget *widget, gpointer data)
 
 	profile_filename = gcm_profile_get_filename (profile);
 	egg_debug ("changed picker space %s", profile_filename);
+
 	gcm_picker_refresh_results ();
 out:
 	if (profile != NULL)
@@ -474,7 +538,7 @@ main (int argc, char *argv[])
 	GtkWidget *main_window;
 	GtkWidget *widget;
 	guint xid = 0;
-	GcmColorimeter *colorimeter = NULL;
+	GcmSensorClient *sensor_client = NULL;
 
 	const GOptionEntry options[] = {
 		{ "parent-window", 'p', 0, G_OPTION_ARG_INT, &xid,
@@ -493,9 +557,7 @@ main (int argc, char *argv[])
 	g_type_init ();
 
 	/* setup LCMS */
-	cmsSetErrorHandler (gcm_picker_lcms_error_cb);
-	cmsErrorAction (LCMS_ERROR_SHOW);
-	cmsSetLanguage ("en", "US");
+	cmsSetLogErrorHandler (gcm_picker_error_cb);
 
 	context = g_option_context_new (NULL);
 	/* TRANSLATORS: tool that is used to pick colors */
@@ -543,9 +605,12 @@ main (int argc, char *argv[])
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 	                                   GCM_DATA G_DIR_SEPARATOR_S "icons");
 
+	/* create a last sample */
+	//gcm_color_xyz_clear (&last_sample);
+
 	/* use the color device */
-	colorimeter = gcm_colorimeter_new ();
-	g_signal_connect (colorimeter, "changed", G_CALLBACK (gcm_picker_colorimeter_changed_cb), NULL);
+	sensor_client = gcm_sensor_client_new ();
+	g_signal_connect (sensor_client, "changed", G_CALLBACK (gcm_picker_sensor_client_changed_cb), NULL);
 
 	/* set the parent window if it is specified */
 	if (xid != 0) {
@@ -571,10 +636,11 @@ main (int argc, char *argv[])
 	gtk_box_pack_start (GTK_BOX(widget), info_bar_hardware, FALSE, FALSE, 0);
 
 	/* disable some ui if no hardware */
-	gcm_picker_colorimeter_setup_ui (colorimeter);
+	gcm_picker_sensor_client_setup_ui (sensor_client);
 
 	/* maintain a list of profiles */
 	profile_store = gcm_profile_store_new ();
+	gcm_profile_store_search (profile_store, GCM_PROFILE_STORE_SEARCH_ALL);
 
 	/* default to AdobeRGB */
 	profile_filename = "/usr/share/color/icc/Argyll/ClayRGB1998.icm";
@@ -587,7 +653,7 @@ main (int argc, char *argv[])
 			  G_CALLBACK (gcm_prefs_space_combo_changed_cb), NULL);
 
 	/* setup results expander */
-	gcm_picker_refresh_results ();
+//	gcm_picker_refresh_results (NULL);
 
 	/* setup initial preview window */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "image_preview"));
@@ -601,8 +667,8 @@ out:
 	g_object_unref (application);
 	if (profile_store != NULL)
 		g_object_unref (profile_store);
-	if (colorimeter != NULL)
-		g_object_unref (colorimeter);
+	if (sensor_client != NULL)
+		g_object_unref (sensor_client);
 	if (calibrate != NULL)
 		g_object_unref (calibrate);
 	if (builder != NULL)
