@@ -42,6 +42,7 @@
 #define GCM_SETTINGS_RENDERING_INTENT_SOFTPROOF		"rendering-intent-softproof"
 #define GCM_SETTINGS_COLORSPACE_RGB			"colorspace-rgb"
 #define GCM_SETTINGS_COLORSPACE_CMYK			"colorspace-cmyk"
+#define GCM_SETTINGS_COLORSPACE_GRAY			"colorspace-gray"
 #define GCM_SETTINGS_USE_PROFILES_FROM_VOLUMES		"use-profiles-from-volumes"
 #define GCM_SETTINGS_CALIBRATION_LENGTH			"calibration-length"
 #define GCM_SETTINGS_SHOW_FINE_TUNING			"show-fine-tuning"
@@ -63,7 +64,7 @@
 #define GCM_PREFS_PACKAGE_NAME_COLOR_PROFILES_EXTRA	"shared-color-profiles-extra"
 
 /* libnotify dummy code */
-#ifndef HAVE_LIBNOTIFY
+#ifndef HAVE_NOTIFY
 #define	notify_init(f1)						/* nothing */
 #define	notify_is_initted(f1)					FALSE
 #define	notify_notification_close(f1,f2)			TRUE
@@ -83,8 +84,6 @@
 
 gboolean	 gcm_utils_mkdir_for_filename		(const gchar		*filename,
 							 GError			**error);
-gboolean	 gcm_utils_mkdir_with_parents		(const gchar		*filename,
-							 GError			**error);
 gboolean	 gcm_utils_mkdir_and_copy		(GFile			*source,
 							 GFile			*destination,
 							 GError			**error);
@@ -97,15 +96,14 @@ void		 gcm_utils_alphanum_lcase		(gchar			*string);
 void		 gcm_utils_ensure_sensible_filename	(gchar			*string);
 gchar		*gcm_utils_get_default_config_location	(void);
 GcmProfileKind	 gcm_utils_device_kind_to_profile_kind	(GcmDeviceKind		 kind);
-gchar		*gcm_utils_format_date_time		(const struct tm	*created);
 gboolean	 gcm_utils_install_package		(const gchar		*package_name,
 							 GtkWindow		*window);
 gboolean	 gcm_utils_is_package_installed		(const gchar		*package_name);
-void		 gcm_utils_ensure_printable		(gchar			*text);
 gboolean	 gcm_utils_is_icc_profile		(GFile			*file);
 gchar		*gcm_utils_linkify			(const gchar		*text);
 const gchar	*gcm_intent_to_localized_text		(GcmIntent	 intent);
 const gchar	*gcm_intent_to_localized_description	(GcmIntent	 intent);
+const gchar	*gcm_colorspace_to_localised_string	(GcmColorspace	 colorspace);
 
 #endif /* __GCM_UTILS_H */
 
