@@ -50,8 +50,18 @@ typedef struct {
 	GtkWidget	*preview_widget_input;
 	GtkWidget	*preview_widget_output;
 	GSettings	*settings;
+	guint		 example_index;
 } GcmViewerPrivate;
 
+typedef enum {
+	GCM_VIEWER_NONE,
+	GCM_VIEWER_CIE_1931,
+	GCM_VIEWER_TRC,
+	GCM_VIEWER_VCGT,
+	GCM_VIEWER_PREVIEW_INPUT,
+	GCM_VIEWER_PREVIEW_OUTPUT
+} GcmViewerGraphType;
+
 enum {
 	GCM_PROFILES_COLUMN_ID,
 	GCM_PROFILES_COLUMN_SORT,
@@ -71,6 +81,7 @@ enum {
 static void gcm_viewer_profile_store_changed_cb (GcmProfileStore *profile_store, GcmViewerPrivate *viewer);
 
 #define GCM_VIEWER_TREEVIEW_WIDTH		350 /* px */
+#define GCM_VIEWER_MAX_EXAMPLE_IMAGES		4
 
 /**
  * gcm_viewer_error_dialog:
@@ -98,6 +109,44 @@ gcm_viewer_close_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 	gtk_application_quit (viewer->application);
 }
 
+/**
+ * gcm_viewer_set_example_image:
+ **/
+static void
+gcm_viewer_set_example_image (GcmViewerPrivate *viewer, GtkImage *image)
+{
+	gchar *filename;
+	filename = g_strdup_printf ("%s/figures/viewer-example-%02i.png", GCM_DATA, viewer->example_index);
+	gtk_image_set_from_file (image, filename);
+	g_free (filename);
+}
+
+/**
+ * gcm_viewer_image_next_cb:
+ **/
+static void
+gcm_viewer_image_next_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
+{
+	viewer->example_index++;
+	if (viewer->example_index == GCM_VIEWER_MAX_EXAMPLE_IMAGES)
+		viewer->example_index = 0;
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_input));
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_output));
+}
+
+/**
+ * gcm_viewer_image_prev_cb:
+ **/
+static void
+gcm_viewer_image_prev_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
+{
+	if (viewer->example_index == 0)
+		viewer->example_index = GCM_VIEWER_MAX_EXAMPLE_IMAGES;
+	viewer->example_index--;
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_input));
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_output));
+}
+
 /**
  * gcm_viewer_preferences_cb:
  **/
@@ -854,12 +903,19 @@ gcm_viewer_set_combo_simple_text (GtkWidget *combo_box)
 static gboolean
 gcm_viewer_startup_phase1_idle_cb (GcmViewerPrivate *viewer)
 {
+	gboolean ret;
 	GtkWidget *widget;
 	GtkTreeSelection *selection;
 	GtkTreePath *path;
+	GcmProfileSearchFlags search_flags = GCM_PROFILE_STORE_SEARCH_ALL;
+
+	/* volume checking is optional */
+	ret = g_settings_get_boolean (viewer->settings, GCM_SETTINGS_USE_PROFILES_FROM_VOLUMES);
+	if (!ret)
+		search_flags &= ~GCM_PROFILE_STORE_SEARCH_VOLUMES;
 
 	/* search the disk for profiles */
-	gcm_profile_store_search (viewer->profile_store, GCM_PROFILE_STORE_SEARCH_ALL);
+	gcm_profile_store_search (viewer->profile_store, search_flags);
 	g_signal_connect (viewer->profile_store, "changed", G_CALLBACK(gcm_viewer_profile_store_changed_cb), viewer);
 
 	/* update list of profiles */
@@ -921,23 +977,31 @@ gcm_viewer_graph_combo_changed_cb (GtkWidget *widget, GcmViewerPrivate *viewer)
 
 	/* hide or show the correct graphs */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_cie_axis"));
-	gtk_widget_set_visible (widget, active == 1);
+	gtk_widget_set_visible (widget, active == GCM_VIEWER_CIE_1931);
 
 	/* hide or show the correct graphs */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_trc_axis"));
-	gtk_widget_set_visible (widget, active == 2);
+	gtk_widget_set_visible (widget, active == GCM_VIEWER_TRC);
 
 	/* hide or show the correct graphs */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_vcgt_axis"));
-	gtk_widget_set_visible (widget, active == 3);
+	gtk_widget_set_visible (widget, active == GCM_VIEWER_VCGT);
 
 	/* hide or show the correct graphs */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_preview_input"));
-	gtk_widget_set_visible (widget, active == 4);
+	gtk_widget_set_visible (widget, active == GCM_VIEWER_PREVIEW_INPUT);
 
 	/* hide or show the correct graphs */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_preview_output"));
-	gtk_widget_set_visible (widget, active == 5);
+	gtk_widget_set_visible (widget, active == GCM_VIEWER_PREVIEW_OUTPUT);
+
+	/* hide or show the buttons */
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "button_image_prev"));
+	gtk_widget_set_visible (widget, (active == GCM_VIEWER_PREVIEW_INPUT) ||
+					(active == GCM_VIEWER_PREVIEW_OUTPUT));
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "button_image_next"));
+	gtk_widget_set_visible (widget, (active == GCM_VIEWER_PREVIEW_INPUT) ||
+					(active == GCM_VIEWER_PREVIEW_OUTPUT));
 
 	/* save to GSettings */
 	g_settings_set_enum (viewer->settings, GCM_SETTINGS_PROFILE_GRAPH_TYPE, active);
@@ -964,8 +1028,8 @@ gcm_viewer_setup_graph_combobox (GcmViewerPrivate *viewer, GtkWidget *widget)
 	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Video card gamma table"));
 
 	/* TRANSLATORS: combo-entry, this is a preview image of what the profile looks like */
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (input)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (output)"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (from sRGB)"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), _("Image preview (to sRGB)"));
 
 	/* get from settings */
 	active = g_settings_get_enum (viewer->settings, GCM_SETTINGS_PROFILE_GRAPH_TYPE);
@@ -1080,6 +1144,14 @@ main (int argc, char **argv)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gcm_viewer_profile_import_cb), viewer);
 
+	/* image next/prev */
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "button_image_next"));
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gcm_viewer_image_next_cb), viewer);
+	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "button_image_prev"));
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gcm_viewer_image_prev_cb), viewer);
+
 	/* hidden until a profile is selected */
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_graph"));
 	gtk_widget_set_visible (widget, FALSE);
@@ -1115,14 +1187,14 @@ main (int argc, char **argv)
 	viewer->preview_widget_input = GTK_WIDGET (gcm_image_new ());
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_preview_input"));
 	gtk_box_pack_end (GTK_BOX(widget), viewer->preview_widget_input, FALSE, FALSE, 0);
-	gtk_image_set_from_file (GTK_IMAGE (viewer->preview_widget_input), GCM_DATA "/figures/viewer-example.png");
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_input));
 	gtk_widget_set_visible (viewer->preview_widget_input, TRUE);
 
 	/* use preview output */
 	viewer->preview_widget_output = GTK_WIDGET (gcm_image_new ());
 	widget = GTK_WIDGET (gtk_builder_get_object (viewer->builder, "vbox_preview_output"));
 	gtk_box_pack_end (GTK_BOX(widget), viewer->preview_widget_output, FALSE, FALSE, 0);
-	gtk_image_set_from_file (GTK_IMAGE (viewer->preview_widget_output), GCM_DATA "/figures/viewer-example.png");
+	gcm_viewer_set_example_image (viewer, GTK_IMAGE (viewer->preview_widget_output));
 	gtk_widget_set_visible (viewer->preview_widget_output, TRUE);
 
 	/* do we set a default size to make the window larger? */
